"""Chinese Simplified translations of the program interface"""

MSG_WELCOME_1 = "欢迎使用 Calcure"
MSG_WELCOME_2 = "您的终端日历和任务管理工具！"
MSG_WELCOME_3 = "配置文件和数据文件已创建于："
MSG_WELCOME_4 = "如需支持、贡献或了解更多信息，请访问："
MSG_WELCOME_5 = "按 ? 查看所有键位绑定，或按任意键继续。"

TITLE_KEYS_GENERAL  = "通用键位绑定"
TITLE_KEYS_CALENDAR = "日历键位绑定"
TITLE_KEYS_JOURNAL  = "任务键位绑定"

KEYS_GENERAL = {
        " Space ": "切换日历和任务视图",
        "   /   ": "切换分屏模式",
        "   *   ": "切换全局隐私模式",
        "   ?   ": "显示此帮助",
        "   Q   ": "重新加载",
        "   q   ": "退出程序",
        }

KEYS_CALENDAR = {
        "  a(A) ": "添加一个（重复）事件",
        "   n   ": "下一个月（天）",
        "   p   ": "上一个月（天）",
        "   x   ": "删除事件",
        "   r   ": "重命名事件",
        "  m(M) ": "移动事件（在本月内）",
        "  g(G) ": "跳转到指定日期（本月内）",
        "   v   ": "切换日视图",
        "   w   ": "切换周视图",
        "   W   ": "显示/隐藏周数",
        "   h   ": "标记事件为高优先级",
        "   l   ": "标记事件为低优先级",
        "   d   ": "标记事件为已完成",
        "   .   ": "切换事件隐私模式",
        "   C   ": "从 Calcurse 导入事件",
        "   R   ": "回到当前月（天）",
        }

KEYS_TODO = {
        "  a(A) ": "新增任务（子任务）",
        "  h(H) ": "标记任务（全部）为高优先级",
        "  l(L) ": "标记任务（全部）为低优先级",
        "  d(D) ": "标记任务（全部）为已完成",
        "  u(U) ": "取消标记任务（全部）",
        "  x(X) ": "删除任务（全部任务及其子任务）",
        "  t(T) ": "为任务开始/暂停计时（删除计时）",
        "   P   ": "切换所有活动计时器",
        "   r   ": "重命名任务",
        "   s   ": "切换任务和子任务",
        "   .   ": "切换任务隐私模式",
        "   -   ": "切换已完成任务的可见性",
        "  f(F) ": "更改（删除）任务截止日期",
        "   m   ": "移动任务",
        "   C   ": "从 Calcurse 导入任务",
        }

MSG_NAME          = "CALCURE"
MSG_VIM           = "方向键和 Vim 键 (j, k, ZZ, ZQ) 也可使用！"
MSG_INFO          = "如需更多信息，请访问："
MSG_SITE          = "https://anufrievroman.gitbook.io/calcure"
MSG_KEYS_INFO     = "自定义快捷键，请参阅："
MSG_KEYS_SITE     = "https://anufrievroman.gitbook.io/calcure/keys"
MSG_EXIT          = "确认退出？(y/n) "

MSG_EVENT_HIGH    = "标记事件为高优先级，事件编号："
MSG_EVENT_LOW     = "标记事件为低优先级，事件编号："
MSG_EVENT_DONE    = "标记事件为已完成，事件编号："
MSG_EVENT_RESET   = "重置事件状态，事件编号："
MSG_EVENT_DEL     = "删除事件编号："
MSG_EVENT_REN     = "重命名事件编号："
MSG_NEW_TITLE     = "输入新的标题："
MSG_EVENT_MV      = "移动事件编号："
MSG_EVENT_MV_TO   = "移动事件到 (YYYY/MM/DD)："
MSG_EVENT_MV_TO_D = "移动事件到："
MSG_EVENT_DATE    = "输入日期："
MSG_EVENT_TITLE   = "输入标题："
MSG_EVENT_REP     = "设置事件重复次数："
MSG_EVENT_FR      = "事件重复周期 (d)天, (w)周, (m)月 或 (y)年？"
MSG_EVENT_IMP     = "从 Calcurse 导入事件？(y/n)"
MSG_EVENT_PRIVACY = "切换事件隐私模式，事件编号："
MSG_TM_ADD        = "为任务编号添加/暂停计时："
MSG_TM_RESET      = "移除任务编号的计时器："
MSG_TS_HIGH       = "标记任务为高优先级，任务编号："
MSG_TS_LOW        = "标记任务为低优先级，任务编号："
MSG_TS_RES        = "重置任务状态，任务编号："
MSG_TS_DONE       = "标记任务为已完成，任务编号："
MSG_TS_DEL        = "删除任务编号："
MSG_TS_DEL_ALL    = "确认删除所有任务？(y/n)"
MSG_TS_EDT_ALL    = "确认执行此操作？(y/n)"
MSG_TS_MOVE       = "移动任务编号："
MSG_TS_MOVE_TO    = "移动任务到编号："
MSG_TS_EDIT       = "编辑任务编号："
MSG_TS_TOG        = "切换子任务编号："
MSG_TS_SUB        = "为任务编号添加子任务："
MSG_TS_TITLE      = "输入子任务："
MSG_TS_IM         = "从 Calcurse 导入任务？(y/n)"
MSG_TS_TW         = "从 Taskwarrior 导入任务？(y/n)"
MSG_TS_NOTHING    = "暂无计划任务..."
MSG_TS_DONE_COUNT = "已完成的任务"
MSG_TS_PRIVACY    = "切换任务隐私模式，任务编号："
MSG_TS_DEAD_ADD   = "为任务编号添加截止日期："
MSG_TS_DEAD_DEL   = "移除任务编号的截止日期："
MSG_TS_DEAD_DATE  = "设置截止日期 (YYYY/MM/DD)："
MSG_WEATHER       = "正在加载天气信息..."
MSG_ERRORS        = "发生错误。查看配置文件夹中的 info.log。"
MSG_INPUT         = "输入有误。"
MSG_GOTO          = "跳转至日期 (YYYY/MM/DD)："
MSG_GOTO_D        = "跳转至日期："

CALENDAR_HINT     = "Space · 切换到任务栏   a · 新增事件   n/p · 切换月份   ? · 所有键位绑定"
CALENDAR_HINT_D   = "Space · 切换到任务栏   a · 新增事件   n/p · 切换日期   ? · 所有键位绑定"
CALENDAR_HINT_W   = "Space · 切换到任务栏   a · 新增事件   n/p · 切换周   ? · 所有键位绑定"
JOURNAL_HINT      = "Space · 切换到日历栏   a · 新增任务   d · 完成   i · 重要   ? · 所有键位绑定"

DAYS = ["周一", "周二", "周三", "周四", "周五", "周六", "周日"]
DAYS_PERSIAN = ["SHANBEH", "YEKSHANBEH", "DOSHANBEH", "SESHANBEH", "CHAHARSHANBEH", "PANJSHANBEH", "JOMEH"]

MONTHS = ["一月", "二月", "三月", "四月", "五月", "六月", "七月", "八月", "九月", "十月", "十一月", "十二月"]
MONTHS_PERSIAN = ["FARVARDIN", "ORDIBEHESHT", "KHORDAD", "TIR", "MORDAD", "SHAHRIVAR", "MEHR", "ABAN", "AZAR", "DEY", "BAHMAN", "ESFAND"]



"""Hungarian translations of the program interface"""

MSG_WELCOME_1 = "Üdvözöl a Calcure"
MSG_WELCOME_2 = "Terminálos naptár- és feladatkezelőd!"
MSG_WELCOME_3 = "A konfigurációs és adatfájlok létrejöttek itt:"
MSG_WELCOME_4 = "Támogatásért, hozzájárulásért és további információért látogasd meg:"
MSG_WELCOME_5 = "Nyomj ?-t a gyorsbillentyűk listájához, vagy bármely más gombot a folytatáshoz."

TITLE_KEYS_GENERAL = "ÁLTALÁNOS BILLENTYŰK"
TITLE_KEYS_CALENDAR = "NAPTÁR BILLENTYŰK"
TITLE_KEYS_JOURNAL  = "NAPLÓ BILLENTYŰK"

KEYS_GENERAL = {
        " Space ": "Váltás a naptár és napló között",
        "   /   ": "Képernyő felosztás be-/kikapcsolása",
        "   *   ": "Adatvédelmi mód be-/kikapcsolása",
        "   ?   ": "Súgó megjelenítése",
        "   Q   ": "Újratöltés",
        "   q   ": "Kilépés",
        }

KEYS_CALENDAR = {
        "  a(A) ": "Új (ismétlődő) esemény hozzáadása",
        "   n   ": "Következő hónap (nap)",
        "   p   ": "Előző hónap (nap)",
        "   x   ": "Esemény törlése",
        "   r   ": "Esemény átnevezése",
        "  m(M) ": "Esemény áthelyezése (ebben a hónapban)",
        "  g(G) ": "Ugrás egy napra (ebben a hónapban)",
        "   v   ": "Napi nézet váltása",
        "   w   ": "Heti nézet váltása",
        "   W   ": "Hét számok megjelenítése",
        "   h   ": "Esemény magas prioritásúvá tétele",
        "   l   ": "Esemény alacsony prioritásúvá tétele",
        "   d   ": "Esemény késznek jelölése",
        "   .   ": "Esemény privátként jelölése",
        "   C   ": "Események importálása Calcurse-ból",
        "   R   ": "Vissza az aktuális hónaphoz (naphoz)",
        }

KEYS_TODO = {
        "  a(A) ": "Új (al)feladat hozzáadása",
        "  h(H) ": "Egy (vagy minden) feladat magas prioritásúvá tétele",
        "  l(L) ": "Egy (vagy minden) feladat alacsony prioritásúvá tétele",
        "  d(D) ": "Egy (vagy minden) feladat késznek jelölése",
        "  u(U) ": "Egy (vagy minden) feladat kész jelölésének törlése",
        "  x(X) ": "Egy (vagy minden) feladat törlése (az összes alfeladattal együtt)",
        "  t(T) ": "Időzítő indítása/szüneteltetése (vagy törlése) feladathoz",
        "   P   ": "Összes aktív időzítő váltása",
        "   r   ": "Feladat átnevezése",
        "   s   ": "Váltás feladat/alfeladat között",
        "   .   ": "Feladat privátként jelölése",
        "   -   ": "Befejezett feladatok láthatóságának váltása",
        "  f(F) ": "Határidő módosítása (hozzáadása/törlése)",
        "   m   ": "Feladat áthelyezése",
        "   C   ": "Feladatok importálása Calcurse-ból",
        }

MSG_NAME          = "CALCURE"
MSG_VIM           = "A nyilak és Vim gombok (j, k, ZZ, ZQ) is működnek!"
MSG_INFO          = "További információért látogasd meg:"
MSG_SITE          = "https://anufrievroman.gitbook.io/calcure"
MSG_KEYS_INFO     = "A billentyűk testreszabásához lásd:"
MSG_KEYS_SITE     = "https://anufrievroman.gitbook.io/calcure/keys"
MSG_EXIT          = "Valóban kilépsz? (y/n) "

MSG_EVENT_HIGH    = "Esemény magas prioritásúvá tétele (sorszám): "
MSG_EVENT_LOW     = "Esemény alacsony prioritásúvá tétele (sorszám): "
MSG_EVENT_DONE    = "Esemény késznek jelölése (sorszám): "
MSG_EVENT_RESET   = "Esemény státuszának visszaállítása (sorszám): "
MSG_EVENT_DEL     = "Esemény törlése (sorszám): "
MSG_EVENT_REN     = "Esemény átnevezése (sorszám): "
MSG_NEW_TITLE     = "Új cím megadása: "
MSG_EVENT_MV      = "Esemény áthelyezése (sorszám): "
MSG_EVENT_MV_TO   = "Áthelyezés ide (ÉÉÉÉ/HH/NN): "
MSG_EVENT_MV_TO_D = "Áthelyezés ide: "
MSG_EVENT_DATE    = "Dátum megadása: "
MSG_EVENT_TITLE   = "Cím megadása: "
MSG_EVENT_REP     = "Hányszor ismétlődjön az esemény: "
MSG_EVENT_FR      = "Ismétlődjön az esemény minden (d)nap, (w)hét, (b)kéthetente, (m)hónap vagy (y)év? "
MSG_EVENT_IMP     = "Események importálása Calcurse-ból? (y/n)"
MSG_EVENT_PRIVACY = "Esemény privátként jelölése (sorszám): "
MSG_TM_ADD        = "Időzítő hozzáadása/szüneteltetése feladathoz (sorszám): "
MSG_TM_RESET      = "Időzítő eltávolítása feladathoz (sorszám): "
MSG_TS_HIGH       = "Feladat magas prioritásúvá tétele (sorszám): "
MSG_TS_LOW        = "Feladat alacsony prioritásúvá tétele (sorszám): "
MSG_TS_RES        = "Feladat státuszának visszaállítása (sorszám): "
MSG_TS_DONE       = "Feladat késznek jelölése (sorszám): "
MSG_TS_DEL        = "Feladat törlése (sorszám): "
MSG_TS_DEL_ALL    = "Valóban minden feladatot törölni? (y/n)"
MSG_TS_EDT_ALL    = "Megerősíted a műveletet? (y/n)"
MSG_TS_MOVE       = "Feladat áthelyezése (sorszám): "
MSG_TS_MOVE_TO    = "Feladat áthelyezése ide (sorszám): "
MSG_TS_EDIT       = "Feladat szerkesztése (sorszám): "
MSG_TS_TOG        = "Alfeladat váltása (sorszám): "
MSG_TS_SUB        = "Alfeladat hozzáadása (sorszám): "
MSG_TS_TITLE      = "Alfeladat megadása: "
MSG_TS_IM         = "Feladatok importálása Calcurse-ból? (y/n)"
MSG_TS_TW         = "Feladatok importálása Taskwarriorból? (y/n)"
MSG_TS_NOTHING    = "Semmi sem szerepel..."
MSG_TS_DONE_COUNT = "Befejezett feladatok"
MSG_TS_PRIVACY    = "Feladat privátként jelölése (sorszám): "
MSG_TS_DEAD_ADD   = "Határidő hozzáadása feladathoz (sorszám): "
MSG_TS_DEAD_DEL   = "Határidő eltávolítása feladathoz (sorszám): "
MSG_TS_DEAD_DATE  = "Határidő hozzáadása (ÉÉÉÉ/HH/NN): "
MSG_WEATHER       = "Időjárás betöltése..."
MSG_ERRORS        = "Hiba történt. Lásd info.log a konfigurációs mappában."
MSG_INPUT         = "Hibás bemenet."
MSG_GOTO          = "Ugrás dátumra (ÉÉÉÉ/HH/NN): "
MSG_GOTO_D        = "Ugrás dátumra: "

CALENDAR_HINT     = "Space · Váltás naplóhoz   a · Esemény hozzáadása  n/p · Hónap váltása   ? · Súgó"
CALENDAR_HINT_D   = "Space · Váltás naplóhoz   a · Esemény hozzáadása  n/p · Nap váltása   ? · Súgó"
CALENDAR_HINT_W   = "Space · Váltás naplóhoz   a · Esemény hozzáadása  n/p · Hét váltása   ? · Súgó"
JOURNAL_HINT      = "Space · Váltás naptárhoz   a · Feladat hozzáadása   d · Kész   i · Fontos   ? · Súgó"

DAYS = ["HÉTFŐ", "KEDD", "SZERDA", "CSÜTÖRTÖK", "PÉNTEK", "SZOMBAT", "VASÁRNAP"]
DAYS_PERSIAN = ["SHANBEH", "YEKSHANBEH", "DOSHANBEH", "SESHANBEH", "CHAHARSHANBEH", "PANJSHANBEH", "JOMEH"]

MONTHS = ["JANUÁR", "FEBRUÁR", "MÁRCIUS", "ÁPRILIS", "MÁJUS", "JÚNIUS", "JÚLIUS", "AUGUSZTUS", "SZEPTEMBER", "OKTÓBER", "NOVEMBER", "DECEMBER"]
MONTHS_PERSIAN = ["FARVARDIN", "ORDIBEHESHT", "KHORDAD", "TIR", "MORDAD", "SHAHRIVAR", "MEHR", "ABAN", "AZAR", "DEY", "BAHMAN", "ESFAND"]



"""Re-implementation of the core calendar library for both Persian and Gregorian styles"""

import enum
import datetime
from itertools import repeat


def convert_to_persian_date(year, month, day):
    """Convert date from Gregorian to Persian calendar"""
    import jdatetime
    persian_date =  jdatetime.date.fromgregorian(day=day, month=month, year=year)
    return persian_date.year, persian_date.month, persian_date.day


def convert_to_gregorian_date(year, month, day):
    """Convert date from Persian to Gregorian calendar"""
    import jdatetime
    gregorian_date = jdatetime.date(year, month, day).togregorian()
    return gregorian_date.year, gregorian_date.month, gregorian_date.day


class Calendar:
    """
    Calendar class, but in contrast to native calendar library, here
    the type of the calendar (Gregorian or Persian) is passed as argument
    and all methods change accordingly.
    """

    def __init__(self, firstweekday, use_persian_calendar):
        self.firstweekday = firstweekday
        self.use_persian_calendar = use_persian_calendar

    def last_day(self, year, month):
        """Return the number of the last day of the month"""
        if self.use_persian_calendar:
            import jdatetime
            isleap = jdatetime.date(year, 1, 1).isleap()
            mdays = [0, 31, 31, 31, 31, 31, 31, 30, 30, 30, 30, 30, 29]
            ndays = mdays[month] + (month == 12 and isleap)
            return ndays
        else:
            isleap = year % 4 == 0 and (year % 100 != 0 or year % 400 == 0)
            mdays = [0, 31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31]
            ndays = mdays[month] + (month == 2 and isleap)
            return ndays

    def first_day(self, year, month):
        """Return weekday of the first day of the month"""
        if self.use_persian_calendar:
            import jdatetime
            return jdatetime.date(year, month, 1).weekday()
        return datetime.date(year, month, 1).weekday()

    def itermonthdays(self, year, month):
        """Iterate through the days of the month"""
        first_day = self.first_day(year, month)
        days_before = (first_day - self.firstweekday) % 7
        yield from repeat(0, days_before)
        yield from range(1, self.last_day(year, month) + 1)
        days_after = (self.firstweekday - first_day - self.last_day(year, month)) % 7
        yield from repeat(0, days_after)

    def monthdayscalendar(self, year, month):
        """Return a matrix representing a month's calendar"""
        days = list(self.itermonthdays(year, month))
        return [days[i:i + 7] for i in range(0, len(days), 7)]

    def week_number(self, year, month, day):
        """Return the week number for a given date"""
        if self.use_persian_calendar:
            import jdatetime
            date = jdatetime.date(year, month, day)
            # For Persian calendar, calculate week number based on year start
            year_start = jdatetime.date(year, 1, 1)
            days_since_start = (date - year_start).days
            week_num = (days_since_start // 7) + 1
            return week_num
        else:
            import datetime
            date = datetime.date(year, month, day)
            # ISO week number (standard week numbering)
            return date.isocalendar()[1]

    def month_week_numbers(self, year, month):
        """Return list of week numbers for each week in the month"""
        weeks = self.monthdayscalendar(year, month)
        week_numbers = []
        for week in weeks:
            # Find the first non-zero day in the week to get the week number
            for day in week:
                if day != 0:
                    week_numbers.append(self.week_number(year, month, day))
                    break
            else:
                # If all days are 0 (shouldn't happen), use previous week + 1
                week_numbers.append(week_numbers[-1] + 1 if week_numbers else 1)
        return week_numbers




import curses

def main(stdscr):
    curses.curs_set(1)
    stdscr.clear()
    stdscr.addstr("Enter characters (ESC to exit):\n")

    while True:
        ch = stdscr.get_wch()  # This reads a wide character (Unicode)

        if isinstance(ch, str):
            if ord(ch) == 27:  # ESC key
                break
            stdscr.addstr(f"Input: {ch} -> Unicode: {ord(ch)}\n")
        else:
            stdscr.addstr(f"Special key code: {ch}\n")

        stdscr.refresh()

curses.wrapper(main)



"""Module provides datatypes used in the program"""

import datetime
import logging
import time
import enum

from dateutil.rrule import rruleset, rrulestr
from calcure.calendars import Calendar


class AppState(enum.Enum):
    """Possible focus states of the application"""
    CALENDAR = 1
    JOURNAL = 2
    HELP = 3
    EXIT = 4
    WELCOME = 5


class CalState(enum.Enum):
    """Possible states of the calendar view"""
    MONTHLY = 1
    DAILY = 2
    WEEKLY = 3


class Status(enum.Enum):
    """Status of events and tasks"""
    NORMAL = 1
    DONE = 2
    IMPORTANT = 3
    UNIMPORTANT = 4


class Frequency(enum.Enum):
    """Frequency of repetitions of recurring events"""
    ONCE = 1
    DAILY = 2
    WEEKLY = 3
    BIWEEKLY = 4
    MONTHLY = 5
    YEARLY = 6


class Task:
    """Tasks created by the user"""

    def __init__(self, item_id, name, status, timer, privacy, year=0, month=0, day=0, calendar_number=None):
        self.item_id = item_id
        self.name = name
        self.status = status
        self.timer = timer
        self.privacy = privacy
        self.year = year
        self.month = month
        self.day = day
        self.calendar_number = calendar_number


class Event:
    """Parent class of all events"""

    def __init__(self, year, month, day, name):
        self.year = year
        self.month = month
        self.day = day
        self.name = name


class UserEvent(Event):
    """Events created by the user"""

    def __init__(self, item_id, year, month, day, name, repetition, frequency, status, privacy,
                                                    calendar_number=None, hour=None, minute=None, rrule=None, exdate=None):
        super().__init__(year, month, day, name)
        self.item_id = item_id
        self.repetition = repetition
        self.frequency = frequency
        self.status = status
        self.privacy = privacy
        self.calendar_number = calendar_number
        self.hour = hour
        self.minute = minute
        self.rrule = rrule
        self.exdate = exdate

    def getDatetime(self):
        local_timezone = datetime.datetime.now(datetime.timezone.utc).astimezone().tzinfo
        return datetime.datetime(self.year, self.month, self.day, self.hour or 0, self.minute or 0, tzinfo=local_timezone)


class UserRepeatedEvent(Event):
    """Events that are repetitions of the original user events"""

    def __init__(self, item_id, year, month, day, name, status, privacy, calendar_number=None):
        super().__init__(year, month, day, name)
        self.item_id = item_id
        self.status = status
        self.privacy = privacy
        self.calendar_number = calendar_number


class Timer:
    """Timer for tasks"""

    def __init__(self, stamps):
        self.stamps = stamps

    @property
    def is_counting(self):
        """Evaluate if the timer is currently running"""
        return False if not self.stamps else (len(self.stamps)%2 == 1)

    @property
    def is_started(self):
        """Evaluate whether the timer has started"""
        return True if self.stamps else False

    @property
    def passed_time(self):
        """Calculate how much time has passed in the un-paused intervals"""
        time_passed = 0

        # Calculate passed time, assuming that even timestamps are pauses:
        for index, _ in enumerate(self.stamps):
            if index > 0 and index % 2 == 1:
                time_passed += float(self.stamps[index]) - float(self.stamps[index-1])

        # Add time passed during the current run:
        if self.is_counting:
            time_passed += time.time() - float(self.stamps[-1])

        # Depending on how much time has passed, show in different formats:
        one_hour = 60*60.0
        one_day = 24*one_hour
        if time_passed < one_hour:
            format_string = "%M:%S"
        else:
            format_string = "%H:%M:%S"
        time_string = str(time.strftime(format_string, time.gmtime(int(time_passed))))

        if 2*one_day > time_passed > one_day:
            time_string = "1 day " + time_string
        if time_passed >= 2*one_day:
            time_string = str(int(time_passed//one_day)) + " days " + time_string
        return time_string


class Collection:
    """Parent class for collections of items like tasks or events"""

    def __init__(self):
        self.items = []
        self.changed = False

    def add_item(self, item):
        """Add an item to the collection"""
        if 1000 > len(item.name) > 0 and item.name != r"\[":
            self.items.append(item)
            self.changed = True

    def delete_item(self, selected_task_id):
        """Delete an item with provided id from the collection"""
        for item in self.items:
            if item.item_id == selected_task_id:
                self.items.remove(item)
                self.changed = True
                break

    def rename_item(self, selected_task_id, new_name):
        """Edit an item name in the collection"""
        for item in self.items:
            if item.item_id == selected_task_id and len(new_name) > 0:
                item.name = new_name
                self.changed = True

    def toggle_item_status(self, selected_task_id, new_status):
        """Toggle the status for the item with provided id"""
        for item in self.items:
            if item.item_id == selected_task_id:
                if item.status == new_status:
                    item.status = Status.NORMAL
                else:
                    item.status = new_status
                self.changed = True
                break

    def toggle_item_privacy(self, selected_task_id):
        """Toggle the privacy for the item with provided id"""
        for item in self.items:
            if item.item_id == selected_task_id:
                item.privacy = not item.privacy
                self.changed = True
                break

    def item_exists(self, item_name):
        """Check if such item already exists in collection"""
        for item in self.items:
            if item.name == item_name:
                return True
        return False

    def change_all_statuses(self, new_status):
        """Change statuses of all items"""
        for item in self.items:
            item.status = new_status
            self.changed = True

    def delete_all_items(self):
        """Delete all items from the collection"""
        self.items.clear()
        self.changed = True

    def is_empty(self):
        """Check if the collection is empty"""
        return len(self.items) == 0

    def is_valid_number(self, number):
        """Check if input is valid and corresponds to an item"""
        if number is None:
            return False
        return 0 <= number < len(self.items)

    def filter_events_that_day(self, screen):
        """Filter only events that happen on the particular day"""
        events_of_the_day = Events()
        for event in self.items:
            if (event.year == screen.year
                and event.month == screen.month
                and event.day == screen.day):
                events_of_the_day.add_item(event)
        return events_of_the_day

    def filter_events_that_month(self, screen):
        """Filter only events that happen on the particular month and sort them by day"""
        events_of_the_month = Events()
        for event in self.items:
            if event.month == screen.month and event.year == screen.year:
                events_of_the_month.add_item(event)
        events_of_the_month.items = sorted(events_of_the_month.items, key=lambda item: item.day)
        return events_of_the_month


class Tasks(Collection):
    """List of tasks created by the user"""

    def __init__(self):
        super().__init__()
        self.paused_by_toggle = set()
        self.done_hidden = False

    @property
    def visible_items(self):
        if not self.done_hidden:
            return self.items
        return [t for t in self.items if t.status != Status.DONE]

    def is_valid_number(self, number):
        if number is None:
            return False
        return 0 <= number < len(self.visible_items)

    @property
    def has_active_timer(self):
        for item in self.items:
            if item.timer.is_counting:
                return True
        return False

    def add_subtask(self, task, parent_id):
        """Add a subtask for certain task in the journal"""
        for idx, item in enumerate(self.items):
            if item.item_id == parent_id:
                level = '----' if item.name[:2] == '--' else '--'
                task.name = level + task.name
                if 100 > len(task.name) > 0:
                    self.items.insert(idx + 1, task)
                    self.changed = True
                break

    def add_timestamp_for_task(self, selected_task_id):
        """Add a timestamp to this task"""
        for item in self.items:
            if item.item_id == selected_task_id:
                item.timer.stamps.append(int(time.time()))
                self.changed = True
                break

    def pause_all_other_timers(self, selected_task_id):
        """Add a timestamp to this task"""
        for item in self.items:
            if item.timer.is_counting and item.item_id != selected_task_id:
                item.timer.stamps.append(int(time.time()))
                self.changed = True

    def toggle_all_timers(self):
        """Pause all running timers, or resume those previously paused by this toggle"""
        if self.paused_by_toggle:
            for item in self.items:
                if item.item_id in self.paused_by_toggle:
                    item.timer.stamps.append(int(time.time()))
                    self.changed = True
            self.paused_by_toggle.clear()
        else:
            for item in self.items:
                if item.timer.is_counting:
                    self.paused_by_toggle.add(item.item_id)
                    item.timer.stamps.append(int(time.time()))
                    self.changed = True

    def reset_timer_for_task(self, selected_task_id):
        """Reset the timer for one of the tasks"""
        for item in self.items:
            if item.item_id == selected_task_id:
                item.timer.stamps = []
                self.changed = True
                break

    def change_deadline(self, selected_task_id, new_year, new_month, new_day):
        """Reset the timer for one of the tasks"""
        for item in self.items:
            if item.item_id == selected_task_id:
                item.year = new_year
                item.month = new_month
                item.day = new_day
                self.changed = True
                break

    def toggle_subtask_state(self, selected_task_id):
        """Toggle the state of the task-subtask"""
        for item in self.items:
            if item.item_id == selected_task_id:
                if item.name[:2] == '--':
                    item.name = item.name[2:]
                else:
                    item.name = '--' + item.name
                self.changed = True

    def move_task(self, number_from, number_to):
        """Move task from certain place to another in the list"""
        visible = self.visible_items
        from_idx = self.items.index(visible[number_from])
        to_idx = self.items.index(visible[number_to])
        self.items.insert(to_idx, self.items.pop(from_idx))
        self.changed = True

    def generate_id(self):
        """Generate a id for a new item. The id is generated as maximum of existing ids plus one"""
        if self.is_empty():
            return 0
        return max([item.item_id for item in self.items]) + 1


class Events(Collection):
    """List of events created by the user or imported"""

    def event_exists(self, new_event):
        """Check if such event already exists in collection"""
        for event in self.items:
            if (event.name == new_event.name
                and event.year == new_event.year
                and event.month == new_event.month
                and event.day == new_event.day):
                return True
        return False

    def change_day(self, selected_item_id, new_day):
        """Move an event to another day within this month"""
        for item in self.items:
            if item.item_id == selected_item_id:
                item.day = new_day
                self.changed = True
                break

    def change_date(self, selected_item_id, new_year, new_month, new_day):
        """Move an event to another date"""
        for item in self.items:
            if item.item_id == selected_item_id:
                item.year = new_year
                item.month = new_month
                item.day = new_day
                self.changed = True
                break

class Birthdays(Events):
    """List of birthdays imported from abook"""

    def filter_events_that_day(self, screen):
        """Filter only birthdays that happen on the particular day"""
        events_of_the_day = Events()
        for event in self.items:
            if event.month == screen.month and event.day == screen.day:
                events_of_the_day.add_item(event)
        return events_of_the_day


class RepeatedEvents(Events):
    """List of events that are repetitions of main events"""

    def __init__(self, user_events, use_persian_calendar, current_year):
        super().__init__()
        self.user_events = user_events
        self.use_persian_calendar = use_persian_calendar

        for event in self.user_events.items:
            if event.repetition >= 1:
                for rep in range(1, event.repetition):
                    temp_year = event.year + rep*(event.frequency == Frequency.YEARLY)
                    temp_month = event.month + rep*(event.frequency == Frequency.MONTHLY)
                    temp_day = event.day + rep*(event.frequency == Frequency.DAILY) + (7*rep*(event.frequency == Frequency.WEEKLY)) + (14*rep*(event.frequency == Frequency.BIWEEKLY))
                    year, month, day = self.calculate_recurring_events(temp_year, temp_month, temp_day, event.frequency)
                    self.add_item(UserRepeatedEvent(event.item_id, year, month, day, event.name, event.status,
                                                    event.privacy, event.calendar_number))

            elif event.rrule:
                dtstart = event.getDatetime()

                # For infinitely repeated events, we limit them by end of the next year:
                if 'COUNT' not in event.rrule and 'UNTIL' not in event.rrule:
                    until_year = current_year + 1
                    until_month = 12
                    event.rrule += ';UNTIL=' + datetime.datetime(until_year, until_month, 1).strftime('%Y%m%dT%H%M%SZ')

                # Create a list of dates of repeated events:
                try:
                    rule = rrulestr(event.rrule, dtstart=dtstart)
                except ValueError as e:
                    logging.error("Problem occurred with event: '%s'.", event.name)
                    continue
                rset = rruleset()
                rset.rrule(rule)

                if event.exdate:
                    exdates_list = [event.exdate] if not isinstance(event.exdate, list) else event.exdate

                    for exdates in exdates_list:
                        for exdate in exdates.dts:
                            exdate_dt = datetime.datetime.combine(exdate.dt, datetime.time.min, tzinfo=dtstart.tzinfo) if not isinstance(exdate.dt, datetime.datetime) else exdate.dt
                            rset.exdate(exdate_dt)

                # Create an event for each repetition and add to the list:
                for date in list(rset)[1:]:
                    self.add_item(UserRepeatedEvent(event.item_id, date.year, date.month, date.day, event.name,
                                                    event.status, event.privacy, event.calendar_number))

    def calculate_recurring_events(self, year, month, day, frequency):
        """Calculate the date of recurring events so that they occur in the next month or year"""
        new_day = day
        new_month = month
        new_year = year
        skip_days = 0

        # Weekly and daily recurrence:
        if frequency in [Frequency.WEEKLY, Frequency.BIWEEKLY, Frequency.DAILY]:

            # Calculate how many days and month to skip to next event:
            for i in range(1000):
                if month + i > 12:
                    year = year + 1
                    month = month - 12

                last_day = Calendar(0, self.use_persian_calendar).last_day(year, month+i)
                if day > skip_days + last_day:
                    skip_days += last_day
                    skip_months = i + 1
                else:
                    skip_months = i
                    break
            new_day = day - skip_days
            new_month = month + skip_months
            new_year = year

        # Monthly recurrence:
        if frequency == Frequency.MONTHLY:
            if month > 12:
                new_year = year + (month - 1)//12
                new_month = month - 12*(new_year - year)
        return new_year, new_month, new_day



"""Keybindings loaded from keybindings.ini or falling back to defaults.

To customise, create ~/.config/calcure/keybindings.ini with a [Keybindings]
section and set any of the keys listed below, e.g.:

    [Keybindings]
    key_done   = D
    key_delete = BackSpace
"""

import configparser
from pathlib import Path

_keybindings_file = Path.home() / ".config" / "calcure" / "keybindings.ini"

# Default keybindings:

# Navigation
KEY_NEXT              = "n"
KEY_PREV              = "p"
KEY_TODAY             = "R"
KEY_GOTO              = "g"
KEY_GOTO_DAY          = "G"

# View switching
KEY_VIEW_DAILY        = "v"
KEY_VIEW_WEEKLY       = "w"
KEY_VIEW_WEEK_NUMBERS = "W"

# Events and tasks — shared actions
KEY_ADD               = "a"
KEY_ADD_EXTRA         = "A"   # recurring event (calendar) / add subtask (journal)
KEY_IMPORTANT         = "i"   # h is a permanent alias
KEY_LOW               = "l"
KEY_UNMARK            = "u"
KEY_DONE              = "d"   # v is a permanent alias in the journal
KEY_DELETE            = "x"
KEY_EDIT              = "e"   # r is a permanent alias
KEY_MOVE              = "m"
KEY_MOVE_IN_MONTH     = "M"
KEY_PRIVACY_ITEM      = "."

# Journal-only task actions
KEY_TIMER             = "t"
KEY_TIMER_RESET       = "T"
KEY_TIMERS_TOGGLE     = "P"
KEY_SUBTASK_STATE     = "s"
KEY_DEADLINE          = "f"
KEY_DEADLINE_REMOVE   = "F"
KEY_DONE_ALL          = "D"   # V is a permanent alias
KEY_UNMARK_ALL        = "U"
KEY_LOW_ALL           = "L"
KEY_IMPORTANT_ALL     = "I"   # H is a permanent alias
KEY_DELETE_ALL        = "X"
KEY_HIDE_DONE         = "-"

# Global
KEY_PRIVACY           = "*"
KEY_IMPORT            = "C"
KEY_RELOAD            = "Q"
KEY_QUIT              = "q"
KEY_HELP              = "?"
KEY_SWITCH            = " "
KEY_SPLIT             = "/"

# Load overrides from keybindings.ini if it exists:

if _keybindings_file.exists():
    _conf = configparser.ConfigParser()
    _conf.read(_keybindings_file)
    if _conf.has_section("Keybindings"):
        _b = _conf["Keybindings"]
        KEY_NEXT              = _b.get("key_next",              KEY_NEXT)
        KEY_PREV              = _b.get("key_prev",              KEY_PREV)
        KEY_TODAY             = _b.get("key_today",             KEY_TODAY)
        KEY_GOTO              = _b.get("key_goto",              KEY_GOTO)
        KEY_GOTO_DAY          = _b.get("key_goto_day",          KEY_GOTO_DAY)
        KEY_VIEW_DAILY        = _b.get("key_view_daily",        KEY_VIEW_DAILY)
        KEY_VIEW_WEEKLY       = _b.get("key_view_weekly",       KEY_VIEW_WEEKLY)
        KEY_VIEW_WEEK_NUMBERS = _b.get("key_view_week_numbers", KEY_VIEW_WEEK_NUMBERS)
        KEY_ADD               = _b.get("key_add",               KEY_ADD)
        KEY_ADD_EXTRA         = _b.get("key_add_extra",         KEY_ADD_EXTRA)
        KEY_IMPORTANT         = _b.get("key_important",         KEY_IMPORTANT)
        KEY_LOW               = _b.get("key_low",               KEY_LOW)
        KEY_UNMARK            = _b.get("key_unmark",            KEY_UNMARK)
        KEY_DONE              = _b.get("key_done",              KEY_DONE)
        KEY_DELETE            = _b.get("key_delete",            KEY_DELETE)
        KEY_EDIT              = _b.get("key_edit",              KEY_EDIT)
        KEY_MOVE              = _b.get("key_move",              KEY_MOVE)
        KEY_MOVE_IN_MONTH     = _b.get("key_move_in_month",     KEY_MOVE_IN_MONTH)
        KEY_PRIVACY_ITEM      = _b.get("key_privacy_item",      KEY_PRIVACY_ITEM)
        KEY_TIMER             = _b.get("key_timer",             KEY_TIMER)
        KEY_TIMER_RESET       = _b.get("key_timer_reset",       KEY_TIMER_RESET)
        KEY_TIMERS_TOGGLE     = _b.get("key_timers_toggle",     KEY_TIMERS_TOGGLE)
        KEY_SUBTASK_STATE     = _b.get("key_subtask_state",     KEY_SUBTASK_STATE)
        KEY_DEADLINE          = _b.get("key_deadline",          KEY_DEADLINE)
        KEY_DEADLINE_REMOVE   = _b.get("key_deadline_remove",   KEY_DEADLINE_REMOVE)
        KEY_DONE_ALL          = _b.get("key_done_all",          KEY_DONE_ALL)
        KEY_UNMARK_ALL        = _b.get("key_unmark_all",        KEY_UNMARK_ALL)
        KEY_LOW_ALL           = _b.get("key_low_all",           KEY_LOW_ALL)
        KEY_IMPORTANT_ALL     = _b.get("key_important_all",     KEY_IMPORTANT_ALL)
        KEY_DELETE_ALL        = _b.get("key_delete_all",        KEY_DELETE_ALL)
        KEY_HIDE_DONE         = _b.get("key_hide_done",         KEY_HIDE_DONE)
        KEY_PRIVACY           = _b.get("key_privacy",           KEY_PRIVACY)
        KEY_IMPORT            = _b.get("key_import",            KEY_IMPORT)
        KEY_RELOAD            = _b.get("key_reload",            KEY_RELOAD)
        KEY_QUIT              = _b.get("key_quit",              KEY_QUIT)
        KEY_HELP              = _b.get("key_help",              KEY_HELP)
        KEY_SWITCH            = _b.get("key_switch",            KEY_SWITCH)
        KEY_SPLIT             = _b.get("key_split",             KEY_SPLIT)

        # Allow writing "Space" in the ini file for the space character:
        if KEY_SWITCH.lower() == "space":
            KEY_SWITCH = " "

# Aggregate key lists for @block_until_valid_input decorators.
# Fixed keys are navigation/special keys that are never user-configurable:

_FIXED_NAV = ["j", "k", "KEY_UP", "KEY_DOWN", "KEY_LEFT", "KEY_RIGHT", "KEY_HOME", "KEY_BTAB"]
_FIXED_ACT = ["h", "r", "c"]   # permanent aliases: h=important, r=edit, c=noop

CALENDAR_KEYS = list(dict.fromkeys([
    KEY_NEXT, KEY_PREV, KEY_TODAY, KEY_GOTO, KEY_GOTO_DAY,
    KEY_VIEW_DAILY, KEY_VIEW_WEEKLY, KEY_VIEW_WEEK_NUMBERS,
    KEY_ADD, KEY_ADD_EXTRA,
    KEY_IMPORTANT, KEY_LOW, KEY_UNMARK, KEY_DONE, KEY_DELETE, KEY_EDIT,
    KEY_MOVE, KEY_MOVE_IN_MONTH, KEY_PRIVACY_ITEM,
    KEY_IMPORT, KEY_RELOAD, KEY_QUIT, KEY_HELP, KEY_SWITCH, KEY_SPLIT, KEY_PRIVACY,
] + _FIXED_NAV + _FIXED_ACT))

WEEKLY_KEYS = list(dict.fromkeys(CALENDAR_KEYS + ["V"]))

JOURNAL_KEYS = list(dict.fromkeys([
    KEY_ADD, KEY_ADD_EXTRA,
    KEY_IMPORTANT, KEY_LOW, KEY_UNMARK, KEY_DONE, KEY_DELETE, KEY_EDIT,
    KEY_MOVE, KEY_PRIVACY_ITEM,
    KEY_TIMER, KEY_TIMER_RESET, KEY_TIMERS_TOGGLE, KEY_SUBTASK_STATE,
    KEY_DEADLINE, KEY_DEADLINE_REMOVE,
    KEY_DONE_ALL, KEY_UNMARK_ALL, KEY_LOW_ALL, KEY_IMPORTANT_ALL, KEY_DELETE_ALL,
    KEY_HIDE_DONE, KEY_IMPORT,
    KEY_PRIVACY, KEY_RELOAD, KEY_QUIT, KEY_HELP, KEY_SWITCH, KEY_SPLIT,
    "h", "v", "V", "H", "r", "c", "KEY_BTAB",
]))

HELP_KEYS = list(dict.fromkeys([KEY_SWITCH, KEY_HELP, KEY_QUIT, "^[", "\x7f"]))



"""This module contains functions that react to user input on each screen"""

import curses
import importlib
import logging

# Modules:
from calcure.data import *
from calcure.dialogues import *
from calcure.configuration import Config
from calcure.keybindings import *

cf = Config()

# Language:
if cf.LANG == "fr":
    from calcure.translations.fr import *
elif cf.LANG == "ru":
    from calcure.translations.ru import *
elif cf.LANG == "it":
    from calcure.translations.it import *
elif cf.LANG == "br":
    from calcure.translations.br import *
elif cf.LANG == "tr":
    from calcure.translations.tr import *
elif cf.LANG == "zh":
    from calcure.translations.zh import *
elif cf.LANG == "tw":
    from calcure.translations.tw import *
elif cf.LANG == "sk":
    from calcure.translations.sk import *
elif cf.LANG == "de":
    from calcure.translations.de import *
elif cf.LANG == "es":
    from calcure.translations.es import *
elif cf.LANG == "hu":
    from calcure.translations.hu import *
elif cf.LANG == "si":
    from calcure.translations.si import *
else:
    from calcure.translations.en import *


def safe_run(func):
    """Decorator preventing crashes on keyboard interruption and no input"""

    def inner(stdscr, screen, *args, **kwargs):
        try:
            func(stdscr, screen, *args, **kwargs)

        # Handle keyboard interruption with ctrl+c:
        except KeyboardInterrupt:
            confirmed = ask_confirmation(stdscr, MSG_EXIT, cf.ASK_CONFIRMATION_TO_QUIT)
            screen.state = AppState.EXIT if confirmed else screen.state

        # Prevent crash if no input:
        except curses.error:
            pass
    return inner


def block_until_valid_input(accepted_keys):
    """Decorator preventing flickering on no-op keypresses."""
    def decorator(func):
        def inner(stdscr, screen, *args, **kwargs):
            while not screen.selection_mode:
                screen.key = stdscr.getkey()
                if vim_style_exit(stdscr, screen):
                    confirmed = ask_confirmation(stdscr, MSG_EXIT, cf.ASK_CONFIRMATION_TO_QUIT)
                    if confirmed:
                        screen.state = AppState.EXIT
                        break
                elif screen.key in accepted_keys:
                    break
            func(stdscr, screen, *args, **kwargs)
        return inner
    return decorator


@safe_run
@block_until_valid_input(accepted_keys=CALENDAR_KEYS)
def control_monthly_screen(stdscr, screen, user_events, importer):
    """Handle user input on the monthly screen"""

    # If we previously entered the selection mode, now we perform the action:
    if screen.selection_mode:
        screen.selection_mode = False

        # Change event status:
        if screen.key in [KEY_IMPORTANT, "h"]:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_HIGH)
            if user_events.filter_events_that_month(screen).is_valid_number(number):
                event_id = user_events.filter_events_that_month(screen).items[number].item_id
                user_events.toggle_item_status(event_id, Status.IMPORTANT)
        if screen.key == KEY_LOW:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_LOW)
            if user_events.filter_events_that_month(screen).is_valid_number(number):
                event_id = user_events.filter_events_that_month(screen).items[number].item_id
                user_events.toggle_item_status(event_id, Status.UNIMPORTANT)
        if screen.key == KEY_UNMARK:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_RESET)
            if user_events.filter_events_that_month(screen).is_valid_number(number):
                event_id = user_events.filter_events_that_month(screen).items[number].item_id
                user_events.toggle_item_status(event_id, Status.NORMAL)
        if screen.key == KEY_DONE:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_DONE)
            if user_events.filter_events_that_month(screen).is_valid_number(number):
                event_id = user_events.filter_events_that_month(screen).items[number].item_id
                user_events.toggle_item_status(event_id, Status.DONE)

        # Toggle event privacy:
        if screen.key == KEY_PRIVACY_ITEM:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_PRIVACY)
            if user_events.filter_events_that_month(screen).is_valid_number(number):
                event_id = user_events.filter_events_that_month(screen).items[number].item_id
                user_events.toggle_item_privacy(event_id)

        # Delete event:
        if screen.key == KEY_DELETE:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_DEL)
            if user_events.filter_events_that_month(screen).is_valid_number(number):
                event_id = user_events.filter_events_that_month(screen).items[number].item_id
                user_events.delete_item(event_id)

        # Rename event:
        if screen.key in [KEY_EDIT, "r"]:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_REN)
            if user_events.filter_events_that_month(screen).is_valid_number(number):
                event_id = user_events.filter_events_that_month(screen).items[number].item_id
                clear_line(stdscr, screen.y_max-2)
                new_name = input_string(stdscr, screen.y_max-2, 0, MSG_NEW_TITLE, screen.x_max-len(MSG_NEW_TITLE)-2)
                user_events.rename_item(event_id, new_name)

        # Move event:
        if screen.key in [KEY_MOVE, KEY_MOVE_IN_MONTH]:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_MV)
            if user_events.filter_events_that_month(screen).is_valid_number(number):
                event_id = user_events.filter_events_that_month(screen).items[number].item_id
                clear_line(stdscr, screen.y_max-2)
                if screen.key == KEY_MOVE:
                    year, month, day = input_date(stdscr, screen.y_max-2, 0, MSG_EVENT_MV_TO)
                    if screen.is_valid_date(year, month, day):
                        user_events.change_date(event_id, year, month, day)

                if screen.key == KEY_MOVE_IN_MONTH:
                    question = f'{MSG_EVENT_MV_TO_D} {screen.year}/{screen.month}/'
                    day = input_day(stdscr, screen.y_max-2, 0, question)
                    if screen.is_valid_day(day):
                        user_events.change_day(event_id, day)

    # Otherwise, we check for user input:
    else:
        # If we need to select an event, change to selection mode:
        selection_keys = [KEY_IMPORTANT, KEY_LOW, KEY_UNMARK, KEY_DONE, KEY_DELETE,
                          KEY_EDIT, KEY_MOVE, KEY_MOVE_IN_MONTH, KEY_PRIVACY_ITEM, "h", "r", "c"]
        if screen.key in selection_keys and user_events.filter_events_that_month(screen).items:
            screen.selection_mode = True

        # Navigation:
        if screen.key in [KEY_NEXT, "j", "KEY_DOWN", "KEY_RIGHT"]:
            screen.next_month()
        if screen.key in [KEY_PREV, "k", "KEY_UP", "KEY_LEFT"]:
            screen.previous_month()
        if screen.key in ["KEY_HOME", KEY_TODAY]:
            screen.reset_to_today()

        # Handle "g" and "G" as go to selected day:
        if screen.key == KEY_GOTO:
            clear_line(stdscr, screen.y_max-2, 0)
            year, month, day = input_date(stdscr, screen.y_max-2, 0, MSG_GOTO)
            if screen.is_valid_date(year, month, day):
                screen.day = day
                screen.month = month
                screen.year = year
                screen.calendar_state = CalState.DAILY

        if screen.key == KEY_GOTO_DAY:
            clear_line(stdscr, screen.y_max-2, 0)
            question = f"{MSG_GOTO_D} {screen.year}/{screen.month}/"
            day = input_day(stdscr, screen.y_max-2, 0, question)
            if screen.is_valid_date(screen.year, screen.month, day):
                screen.day = day
                screen.calendar_state = CalState.DAILY

        # Change the view:
        if screen.key == KEY_VIEW_DAILY:
            screen.day = 1
            screen.calendar_state = CalState.DAILY
        if screen.key == KEY_VIEW_WEEKLY:
            today = screen.today
            if screen.year == today.year and screen.month == today.month:
                screen.day = today.day
            else:
                screen.day = 1
            screen.calendar_state = CalState.WEEKLY

        # Add single event:
        if screen.key == KEY_ADD:
            question = f'{MSG_EVENT_DATE} {screen.year}/{screen.month}/'
            day = input_day(stdscr, screen.y_max-2, 0, question)
            if screen.is_valid_day(day):
                clear_line(stdscr, screen.y_max-2)
                name = input_string(stdscr, screen.y_max-2, 0, MSG_EVENT_TITLE, screen.x_max-len(MSG_EVENT_TITLE)-2)
                event_id = user_events.items[-1].item_id + 1 if not user_events.is_empty() else 1
                user_events.add_item(UserEvent(event_id, screen.year, screen.month, day, name, 1, Frequency.ONCE, Status.NORMAL, False))

        # Add a recurring event:
        if screen.key == KEY_ADD_EXTRA:
            question = f'{MSG_EVENT_DATE}{screen.year}/{screen.month}/'
            day = input_day(stdscr, screen.y_max-2, 0, question)
            if screen.is_valid_day(day):
                clear_line(stdscr, screen.y_max-2)
                name = input_string(stdscr, screen.y_max-2, 0, MSG_EVENT_TITLE, screen.x_max-len(MSG_EVENT_TITLE)-2)
                item_id = user_events.items[-1].item_id + 1 if not user_events.is_empty() else 1
                reps = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_REP)
                freq = input_frequency(stdscr, screen.y_max-2, 0, MSG_EVENT_FR)
                if reps is not None and freq is not None:
                    reps = 1 if reps == 0 else reps
                    user_events.add_item(UserEvent(item_id, screen.year, screen.month, day, name, reps+1, freq, Status.NORMAL, False))

        # Reload:
        if screen.key == KEY_RELOAD:
            screen.reload_data = True

        # Imports:
        if screen.key == KEY_IMPORT:
            confirmed = ask_confirmation(stdscr, MSG_EVENT_IMP, cf.ASK_CONFIRMATIONS)
            if confirmed:
                importer.import_events_from_calcurse()

        # Other actions:
        if screen.key == KEY_PRIVACY:
            screen.privacy = not screen.privacy
        if screen.key in [KEY_SWITCH, "KEY_BTAB"]:
            screen.state = AppState.JOURNAL
        if screen.key == KEY_HELP:
            screen.state = AppState.HELP
        if screen.key == KEY_QUIT:
            confirmed = ask_confirmation(stdscr, MSG_EXIT, cf.ASK_CONFIRMATION_TO_QUIT)
            screen.state = AppState.EXIT if confirmed else screen.state
        if screen.key == KEY_SPLIT:
            screen.split = not screen.split
        if screen.key == KEY_VIEW_WEEK_NUMBERS:
            screen.show_week_numbers = not screen.show_week_numbers


@safe_run
@block_until_valid_input(accepted_keys=CALENDAR_KEYS)
def control_daily_screen(stdscr, screen, user_events, importer):
    """Handle user input on the daily screen"""
    # If we previously entered the selection mode, now we perform the action:
    if screen.selection_mode:
        screen.selection_mode = False
        view_events = _filter_events_that_daily_view(screen, user_events)

        # Change event status:
        if screen.key in [KEY_IMPORTANT, "h"]:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_HIGH)
            if view_events.is_valid_number(number):
                user_events.toggle_item_status(view_events.items[number].item_id, Status.IMPORTANT)
        if screen.key == KEY_LOW:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_LOW)
            if view_events.is_valid_number(number):
                user_events.toggle_item_status(view_events.items[number].item_id, Status.UNIMPORTANT)
        if screen.key == KEY_UNMARK:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_RESET)
            if view_events.is_valid_number(number):
                user_events.toggle_item_status(view_events.items[number].item_id, Status.NORMAL)
        if screen.key == KEY_DONE:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_DONE)
            if view_events.is_valid_number(number):
                user_events.toggle_item_status(view_events.items[number].item_id, Status.DONE)

        # Toggle event privacy:
        if screen.key == KEY_PRIVACY_ITEM:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_PRIVACY)
            if view_events.is_valid_number(number):
                user_events.toggle_item_privacy(view_events.items[number].item_id)

        # Delete event:
        if screen.key == KEY_DELETE:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_DEL)
            if view_events.is_valid_number(number):
                user_events.delete_item(view_events.items[number].item_id)

        # Rename event:
        if screen.key in [KEY_EDIT, "r"]:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_REN)
            if view_events.is_valid_number(number):
                clear_line(stdscr, screen.y_max-2)
                new_name = input_string(stdscr, screen.y_max-2, 0, MSG_NEW_TITLE, screen.x_max-len(MSG_NEW_TITLE)-2)
                user_events.rename_item(view_events.items[number].item_id, new_name)

        # Move event:
        if screen.key in [KEY_MOVE, KEY_MOVE_IN_MONTH]:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_MV)
            if view_events.is_valid_number(number):
                item_id = view_events.items[number].item_id
                clear_line(stdscr, screen.y_max-2)
                if screen.key == KEY_MOVE:
                    year, month, day = input_date(stdscr, screen.y_max-2, 0, MSG_EVENT_MV_TO)
                    if screen.is_valid_date(year, month, day):
                        user_events.change_date(item_id, year, month, day)
                if screen.key == KEY_MOVE_IN_MONTH:
                    question = f'{MSG_EVENT_MV_TO_D}{screen.year}/{screen.month}/'
                    day = input_day(stdscr, screen.y_max-2, 0, question)
                    if screen.is_valid_day(day):
                        user_events.change_day(item_id, day)

    # Otherwise, we check for user input:
    else:
        # If we need to select an event, change to selection mode:
        selection_keys = [KEY_IMPORTANT, KEY_LOW, KEY_UNMARK, KEY_DONE, KEY_DELETE,
                          KEY_EDIT, KEY_MOVE, KEY_MOVE_IN_MONTH, KEY_PRIVACY_ITEM, "h", "r", "c"]
        if screen.key in selection_keys and _filter_events_that_daily_view(screen, user_events).items:
            screen.selection_mode = True

        # Navigation:
        if screen.key in [KEY_NEXT, "j", "KEY_UP", "KEY_RIGHT"]:
            if cf.INVERSE_DAILY_SCROLL:
                screen.previous_day()
            else:
                screen.next_day()
        if screen.key in [KEY_PREV, "k", "KEY_DOWN", "KEY_LEFT"]:
            if cf.INVERSE_DAILY_SCROLL:
                screen.next_day()
            else:
                screen.previous_day()
        if screen.key in ["KEY_HOME", KEY_TODAY]:
            screen.reset_to_today()

        # Add single event:
        if screen.key == KEY_ADD:
            name = input_string(stdscr, screen.y_max-2, 0, MSG_EVENT_TITLE, screen.x_max-len(MSG_EVENT_TITLE)-2)
            item_id = user_events.items[-1].item_id + 1 if not user_events.is_empty() else 1
            user_events.add_item(UserEvent(item_id, screen.year, screen.month, screen.day, name, 1, Frequency.ONCE, Status.NORMAL, False))

        # Add a recurring event:
        if screen.key == KEY_ADD_EXTRA:
            name = input_string(stdscr, screen.y_max-2, 0, MSG_EVENT_TITLE, screen.x_max-len(MSG_EVENT_TITLE)-2)
            item_id = user_events.items[-1].item_id + 1 if not user_events.is_empty() else 1
            reps = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_REP)
            freq = input_frequency(stdscr, screen.y_max-2, 0, MSG_EVENT_FR)
            if reps is not None and freq is not None:
                reps = 1 if reps == 0 else reps
                user_events.add_item(UserEvent(item_id, screen.year, screen.month, screen.day, name, reps+1, freq, Status.NORMAL, False))

        # Import from calcurse:
        if screen.key == KEY_IMPORT:
            confirmed = ask_confirmation(stdscr, MSG_EVENT_IMP, cf.ASK_CONFIRMATIONS)
            if confirmed:
                importer.import_events_from_calcurse()

        # Handle "g" and "G" as go to selected (prefilled) day:
        if screen.key == KEY_GOTO:
            clear_line(stdscr, screen.y_max-2, 0)
            year, month, day = input_date(stdscr, screen.y_max-2, 0, MSG_GOTO)
            if screen.is_valid_date(year, month, day):
                screen.day = day
                screen.month = month
                screen.year = year
                screen.calendar_state = CalState.DAILY

        if screen.key == KEY_GOTO_DAY:
            clear_line(stdscr, screen.y_max-2, 0)
            question = f"{MSG_GOTO_D} {screen.year}/{screen.month}/"
            day = input_day(stdscr, screen.y_max-2, 0, question)
            if screen.is_valid_date(screen.year, screen.month, day):
                screen.day = day
                screen.calendar_state = CalState.DAILY

        # Reload:
        if screen.key == KEY_RELOAD:
            screen.reload_data = True

        # Change the view:
        if screen.key == KEY_VIEW_DAILY:
            screen.calendar_state = CalState.MONTHLY
        if screen.key == KEY_VIEW_WEEKLY:
            screen.calendar_state = CalState.WEEKLY

        # Other actions:
        if screen.key == KEY_PRIVACY:
            screen.privacy = not screen.privacy
        if screen.key in [KEY_SWITCH, "KEY_BTAB"]:
            screen.state = AppState.JOURNAL
        if screen.key == KEY_HELP:
            screen.state = AppState.HELP
        if screen.key == KEY_QUIT:
            confirmed = ask_confirmation(stdscr, MSG_EXIT, cf.ASK_CONFIRMATION_TO_QUIT)
            screen.state = AppState.EXIT if confirmed else screen.state
        if screen.key == KEY_SPLIT:
            screen.split = not screen.split
        if screen.key == KEY_VIEW_WEEK_NUMBERS:
            screen.show_week_numbers = not screen.show_week_numbers


def _filter_events_that_daily_view(screen, user_events):
    """Return all user events across all days currently visible in the daily view."""
    from calcure.data import Events
    max_num_days = (screen.y_max - 5) // 2
    result = Events()
    orig = (screen.year, screen.month, screen.day)
    for _ in range(max_num_days):
        for evt in user_events.filter_events_that_day(screen).items:
            result.add_item(evt)
        screen.next_day()
    screen.year, screen.month, screen.day = orig
    return result


def _filter_events_that_week(screen, user_events):
    """Return all user events across the 7 days of the current week, in column order."""
    from calcure.data import Events
    week = screen.week_dates(cf.START_WEEK_DAY)
    result = Events()
    orig = (screen.year, screen.month, screen.day)
    for d in week:
        screen.year, screen.month, screen.day = d.year, d.month, d.day
        for evt in user_events.filter_events_that_day(screen).items:
            result.add_item(evt)
    screen.year, screen.month, screen.day = orig
    return result


@safe_run
@block_until_valid_input(accepted_keys=WEEKLY_KEYS)
def control_weekly_screen(stdscr, screen, user_events, importer):
    """Handle user input on the weekly screen"""

    if screen.selection_mode:
        screen.selection_mode = False
        week_events = _filter_events_that_week(screen, user_events)

        # Change event status:
        if screen.key in [KEY_IMPORTANT, "h"]:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_HIGH)
            if week_events.is_valid_number(number):
                user_events.toggle_item_status(week_events.items[number].item_id, Status.IMPORTANT)
        if screen.key == KEY_LOW:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_LOW)
            if week_events.is_valid_number(number):
                user_events.toggle_item_status(week_events.items[number].item_id, Status.UNIMPORTANT)
        if screen.key == KEY_UNMARK:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_RESET)
            if week_events.is_valid_number(number):
                user_events.toggle_item_status(week_events.items[number].item_id, Status.NORMAL)
        if screen.key == KEY_DONE:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_DONE)
            if week_events.is_valid_number(number):
                user_events.toggle_item_status(week_events.items[number].item_id, Status.DONE)

        if screen.key == KEY_PRIVACY_ITEM:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_PRIVACY)
            if week_events.is_valid_number(number):
                user_events.toggle_item_privacy(week_events.items[number].item_id)

        if screen.key == KEY_DELETE:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_DEL)
            if week_events.is_valid_number(number):
                user_events.delete_item(week_events.items[number].item_id)

        if screen.key in [KEY_EDIT, "r"]:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_REN)
            if week_events.is_valid_number(number):
                clear_line(stdscr, screen.y_max-2)
                new_name = input_string(stdscr, screen.y_max-2, 0, MSG_NEW_TITLE, screen.x_max-len(MSG_NEW_TITLE)-2)
                user_events.rename_item(week_events.items[number].item_id, new_name)

        if screen.key in [KEY_MOVE, KEY_MOVE_IN_MONTH]:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_MV)
            if week_events.is_valid_number(number):
                event_id = week_events.items[number].item_id
                clear_line(stdscr, screen.y_max-2)
                if screen.key == KEY_MOVE:
                    year, month, day = input_date(stdscr, screen.y_max-2, 0, MSG_EVENT_MV_TO)
                    if screen.is_valid_date(year, month, day):
                        user_events.change_date(event_id, year, month, day)
                if screen.key == KEY_MOVE_IN_MONTH:
                    question = f'{MSG_EVENT_MV_TO_D} {screen.year}/{screen.month}/'
                    day = input_day(stdscr, screen.y_max-2, 0, question)
                    if screen.is_valid_day(day):
                        user_events.change_day(event_id, day)

    else:
        selection_keys = [KEY_IMPORTANT, KEY_LOW, KEY_UNMARK, KEY_DONE, KEY_DELETE,
                          KEY_EDIT, KEY_MOVE, KEY_MOVE_IN_MONTH, KEY_PRIVACY_ITEM, "h", "r"]
        if screen.key in selection_keys and _filter_events_that_week(screen, user_events).items:
            screen.selection_mode = True

        # Navigation (week by week):
        if screen.key in [KEY_NEXT, "j", "KEY_DOWN", "KEY_RIGHT"]:
            screen.next_week()
        if screen.key in [KEY_PREV, "k", "KEY_UP", "KEY_LEFT"]:
            screen.previous_week()
        if screen.key in ["KEY_HOME", KEY_TODAY]:
            screen.reset_to_today()

        if screen.key == KEY_GOTO:
            clear_line(stdscr, screen.y_max-2, 0)
            year, month, day = input_date(stdscr, screen.y_max-2, 0, MSG_GOTO)
            if screen.is_valid_date(year, month, day):
                screen.year, screen.month, screen.day = year, month, day

        # Switch views:
        if screen.key == KEY_VIEW_DAILY:
            screen.calendar_state = CalState.DAILY
        if screen.key in [KEY_VIEW_WEEKLY, "V"]:
            screen.calendar_state = CalState.MONTHLY

        # Add single event:
        if screen.key == KEY_ADD:
            week = screen.week_dates(cf.START_WEEK_DAY)
            question = f'{MSG_EVENT_DATE} {week[0].year}/{week[0].month}/'
            day = input_day(stdscr, screen.y_max-2, 0, question)
            matched = next((d for d in week if d.day == day), None) if day is not None else None
            if matched:
                screen.year, screen.month, screen.day = matched.year, matched.month, matched.day
                clear_line(stdscr, screen.y_max-2)
                name = input_string(stdscr, screen.y_max-2, 0, MSG_EVENT_TITLE, screen.x_max-len(MSG_EVENT_TITLE)-2)
                event_id = user_events.items[-1].item_id + 1 if not user_events.is_empty() else 1
                user_events.add_item(UserEvent(event_id, screen.year, screen.month, screen.day, name, 1, Frequency.ONCE, Status.NORMAL, False))

        # Add a recurring event:
        if screen.key == KEY_ADD_EXTRA:
            week = screen.week_dates(cf.START_WEEK_DAY)
            question = f'{MSG_EVENT_DATE} {week[0].year}/{week[0].month}/'
            day = input_day(stdscr, screen.y_max-2, 0, question)
            matched = next((d for d in week if d.day == day), None) if day is not None else None
            if matched:
                screen.year, screen.month, screen.day = matched.year, matched.month, matched.day
                clear_line(stdscr, screen.y_max-2)
                name = input_string(stdscr, screen.y_max-2, 0, MSG_EVENT_TITLE, screen.x_max-len(MSG_EVENT_TITLE)-2)
                item_id = user_events.items[-1].item_id + 1 if not user_events.is_empty() else 1
                reps = input_integer(stdscr, screen.y_max-2, 0, MSG_EVENT_REP)
                freq = input_frequency(stdscr, screen.y_max-2, 0, MSG_EVENT_FR)
                if reps is not None and freq is not None:
                    reps = 1 if reps == 0 else reps
                    user_events.add_item(UserEvent(item_id, screen.year, screen.month, screen.day, name, reps+1, freq, Status.NORMAL, False))

        if screen.key == KEY_IMPORT:
            confirmed = ask_confirmation(stdscr, MSG_EVENT_IMP, cf.ASK_CONFIRMATIONS)
            if confirmed:
                importer.import_events_from_calcurse()

        if screen.key == KEY_RELOAD:
            screen.reload_data = True

        if screen.key == KEY_PRIVACY:
            screen.privacy = not screen.privacy
        if screen.key in [KEY_SWITCH, "KEY_BTAB"]:
            screen.state = AppState.JOURNAL
        if screen.key == KEY_HELP:
            screen.state = AppState.HELP
        if screen.key == KEY_QUIT:
            confirmed = ask_confirmation(stdscr, MSG_EXIT, cf.ASK_CONFIRMATION_TO_QUIT)
            screen.state = AppState.EXIT if confirmed else screen.state
        if screen.key == KEY_SPLIT:
            screen.split = not screen.split
        if screen.key == KEY_VIEW_WEEK_NUMBERS:
            screen.show_week_numbers = not screen.show_week_numbers


@safe_run
@block_until_valid_input(accepted_keys=JOURNAL_KEYS)
def control_journal_screen(stdscr, screen, user_tasks, importer):
    """Process user input on the journal screen"""
    # If we previously selected a task, now we perform the action:
    if screen.selection_mode:

        # Timer operations:
        if screen.key == KEY_TIMER:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_TM_ADD)
            if user_tasks.is_valid_number(number):
                task_id = user_tasks.visible_items[number].item_id
                if cf.ONE_TIMER_AT_A_TIME:
                    user_tasks.pause_all_other_timers(task_id)
                user_tasks.add_timestamp_for_task(task_id)

        if screen.key == KEY_TIMER_RESET:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_TM_RESET)
            if user_tasks.is_valid_number(number):
                task_id = user_tasks.visible_items[number].item_id
                user_tasks.reset_timer_for_task(task_id)

        # Add deadline:
        if screen.key == KEY_DEADLINE:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_TS_DEAD_ADD)
            if user_tasks.is_valid_number(number):
                task_id = user_tasks.visible_items[number].item_id
                clear_line(stdscr, screen.y_max-2, 0)
                year, month, day = input_date(stdscr, screen.y_max-2, 0, MSG_TS_DEAD_DATE)
                if screen.is_valid_date(year, month, day):
                    user_tasks.change_deadline(task_id, year, month, day)

        # Remove deadline:
        if screen.key == KEY_DEADLINE_REMOVE:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_TS_DEAD_DEL)
            if user_tasks.is_valid_number(number):
                task_id = user_tasks.visible_items[number].item_id
                user_tasks.change_deadline(task_id, 0, 0, 0)

        # Change the status:
        if screen.key in [KEY_IMPORTANT, "h"]:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_TS_HIGH)
            if user_tasks.is_valid_number(number):
                task_id = user_tasks.visible_items[number].item_id
                user_tasks.toggle_item_status(task_id, Status.IMPORTANT)
        if screen.key == KEY_LOW:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_TS_LOW)
            if user_tasks.is_valid_number(number):
                task_id = user_tasks.visible_items[number].item_id
                user_tasks.toggle_item_status(task_id, Status.UNIMPORTANT)
        if screen.key == KEY_UNMARK:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_TS_RES)
            if user_tasks.is_valid_number(number):
                task_id = user_tasks.visible_items[number].item_id
                user_tasks.toggle_item_status(task_id, Status.NORMAL)
        if screen.key in [KEY_DONE, "v"]:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_TS_DONE)
            if user_tasks.is_valid_number(number):
                task_id = user_tasks.visible_items[number].item_id
                user_tasks.toggle_item_status(task_id, Status.DONE)

        # Toggle task privacy:
        if screen.key == KEY_PRIVACY_ITEM:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_TS_PRIVACY)
            if user_tasks.is_valid_number(number):
                task_id = user_tasks.visible_items[number].item_id
                user_tasks.toggle_item_privacy(task_id)

        # Modify the task:
        if screen.key == KEY_DELETE:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_TS_DEL)
            if user_tasks.is_valid_number(number):
                task_id = user_tasks.visible_items[number].item_id
                user_tasks.delete_item(task_id)
        if screen.key == KEY_MOVE:
            number_from = input_integer(stdscr, screen.y_max-2, 0, MSG_TS_MOVE)
            if user_tasks.is_valid_number(number_from):
                clear_line(stdscr, screen.y_max-2)
                number_to = input_integer(stdscr, screen.y_max-2, 0, MSG_TS_MOVE_TO)
                if user_tasks.is_valid_number(number_to):
                    user_tasks.move_task(number_from, number_to)
        if screen.key in [KEY_EDIT, "r"]:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_TS_EDIT)
            if user_tasks.is_valid_number(number):
                task_id = user_tasks.visible_items[number].item_id
                clear_line(stdscr, number+2, screen.x_min)
                new_name = input_string(stdscr, number+2, screen.x_min, cf.TODO_ICON+' ', screen.x_max-4)
                user_tasks.rename_item(task_id, new_name)

        # Subtask operations:
        if screen.key == KEY_SUBTASK_STATE:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_TS_TOG)
            if user_tasks.is_valid_number(number):
                task_id = user_tasks.visible_items[number].item_id
                user_tasks.toggle_subtask_state(task_id)
        if screen.key == KEY_ADD_EXTRA:
            number = input_integer(stdscr, screen.y_max-2, 0, MSG_TS_SUB)
            if user_tasks.is_valid_number(number):
                clear_line(stdscr, screen.y_max-2, 0)
                task_name = input_string(stdscr, screen.y_max-2, 0, MSG_TS_TITLE, screen.x_max-len(MSG_TS_TITLE)-2)
                task_id = user_tasks.generate_id()
                user_tasks.add_subtask(Task(task_id, task_name, Status.NORMAL, Timer([]), False), user_tasks.visible_items[number].item_id)
        screen.selection_mode = False

    # Otherwise, we check for user input:
    else:
        # If we need to select a task, change to selection mode:
        selection_keys = [KEY_TIMER, KEY_TIMER_RESET, KEY_IMPORTANT, KEY_LOW, KEY_UNMARK, KEY_DONE,
                          KEY_SUBTASK_STATE, KEY_DELETE, KEY_EDIT, KEY_ADD_EXTRA, KEY_MOVE,
                          KEY_PRIVACY_ITEM, KEY_DEADLINE, KEY_DEADLINE_REMOVE, "h", "v", "r", "c"]
        if screen.key in selection_keys and user_tasks.items:
            screen.selection_mode = True

        # Add single task:
        if screen.key == KEY_ADD:
            clear_line(stdscr, len(user_tasks.items) + 2, screen.x_min)
            task_name = input_string(stdscr, len(user_tasks.items) + 2, screen.x_min, cf.TODO_ICON+' ', screen.x_max - 4)
            task_id = user_tasks.generate_id()
            user_tasks.add_item(Task(task_id, task_name, Status.NORMAL, Timer([]), False))

        # Bulk operations:
        if screen.key in ["V", KEY_DONE_ALL]:
            confirmed = ask_confirmation(stdscr, MSG_TS_EDT_ALL, cf.ASK_CONFIRMATIONS)
            if confirmed:
                user_tasks.change_all_statuses(Status.DONE)
        if screen.key == KEY_UNMARK_ALL:
            confirmed = ask_confirmation(stdscr, MSG_TS_EDT_ALL, cf.ASK_CONFIRMATIONS)
            if confirmed:
                user_tasks.change_all_statuses(Status.NORMAL)
        if screen.key == KEY_LOW_ALL:
            confirmed = ask_confirmation(stdscr, MSG_TS_EDT_ALL, cf.ASK_CONFIRMATIONS)
            if confirmed:
                user_tasks.change_all_statuses(Status.UNIMPORTANT)
        if screen.key in [KEY_IMPORTANT_ALL, "H"]:
            confirmed = ask_confirmation(stdscr, MSG_TS_EDT_ALL, cf.ASK_CONFIRMATIONS)
            if confirmed:
                user_tasks.change_all_statuses(Status.IMPORTANT)
        if screen.key == KEY_DELETE_ALL:
            confirmed = ask_confirmation(stdscr, MSG_TS_DEL_ALL, cf.ASK_CONFIRMATIONS)
            if confirmed:
                user_tasks.delete_all_items()

        # Imports:
        if screen.key == KEY_IMPORT:
            confirmed = ask_confirmation(stdscr, MSG_TS_IM, cf.ASK_CONFIRMATIONS)
            if confirmed:
                importer.import_tasks_from_calcurse()

        # Toggle all active timers:
        if screen.key == KEY_TIMERS_TOGGLE:
            user_tasks.toggle_all_timers()

        # Reload:
        if screen.key == KEY_RELOAD:
            screen.reload_data = True

        # Other actions:
        if screen.key == KEY_PRIVACY:
            screen.privacy = not screen.privacy
        if screen.key == KEY_HIDE_DONE:
            user_tasks.done_hidden = not user_tasks.done_hidden
        if screen.key in [KEY_SWITCH, "KEY_BTAB"]:
            screen.state = AppState.CALENDAR
        if screen.key == KEY_HELP:
            screen.state = AppState.HELP
        if screen.key == KEY_QUIT:
            confirmed = ask_confirmation(stdscr, MSG_EXIT, cf.ASK_CONFIRMATION_TO_QUIT)
            screen.state = AppState.EXIT if confirmed else screen.state
        if screen.key == KEY_SPLIT:
            screen.split = not screen.split


@safe_run
@block_until_valid_input(accepted_keys=HELP_KEYS)
def control_help_screen(stdscr, screen):
    """Process user input on the help screen"""
    # Handle keys to exit the help screen:
    if screen.key in [KEY_SWITCH, KEY_HELP, KEY_QUIT, "^[", "\x7f"]:
        screen.state = AppState.CALENDAR


@safe_run
def control_welcome_screen(stdscr, screen):
    """Process user input on the welcome screen"""
    # Getting user's input:
    screen.key = stdscr.getkey()

    # Handle key to call help screen:
    if screen.key == KEY_HELP:
        screen.state = AppState.HELP

    # Otherwise, just start the program:
    else:
        screen.state = AppState.CALENDAR



"""Chinese Traditional translations of the program interface"""

MSG_WELCOME_1 = "歡迎使用 Calcure"
MSG_WELCOME_2 = "您的終端日曆與任務管理器！"
MSG_WELCOME_3 = "配置檔案和資料檔案已創建於："
MSG_WELCOME_4 = "如需支援、貢獻或更多資訊，請造訪："
MSG_WELCOME_5 = "按 ? 查看所有快捷鍵，或按任意鍵繼續。"

TITLE_KEYS_GENERAL = "一般快捷鍵"
TITLE_KEYS_CALENDAR = "日曆快捷鍵"
TITLE_KEYS_JOURNAL  = "日誌快捷鍵"

KEYS_GENERAL = {
        " Space ": "切換日曆和日誌",
        "   /   ": "切換螢幕分割顯示",
        "   *   ": "切換全域隱私模式",
        "   ?   ": "顯示此說明",
        "   Q   ": "重新載入",
        "   q   ": "退出",
        }

KEYS_CALENDAR = {
        "  a(A) ": "新增（重複）行程",
        "   n   ": "下個月（天）",
        "   p   ": "上個月（天）",
        "   x   ": "刪除行程",
        "   r   ": "重新命名行程",
        "  m(M) ": "移動行程（當月）",
        "  g(G) ": "跳至指定日期（當月）",
        "   v   ": "切換每日顯示",
        "   w   ": "切換每週顯示",
        "   W   ": "顯示/隱藏週數",
        "   h   ": "標記行程為高優先級",
        "   l   ": "標記行程為低優先級",
        "   d   ": "標記行程為已完成",
        "   .   ": "切換行程隱私設定",
        "   C   ": "從 Calcurse 匯入行程",
        "   R   ": "回到目前月份（天）",
        }

KEYS_TODO = {
        "  a(A) ": "新增新的（子）任務",
        "  h(H) ": "將單個（所有）任務設為高優先級",
        "  l(L) ": "將單個（所有）任務設為低優先級",
        "  d(D) ": "將單個（所有）任務標記為已完成",
        "  u(U) ": "取消標記單個（所有）任務",
        "  x(X) ": "刪除單個（所有）任務（含所有子任務）",
        "  t(T) ": "為任務開始/暫停（移除）計時器",
        "   P   ": "切換所有活動計時器",
        "   r   ": "重新命名任務",
        "   s   ": "切換主任務與子任務",
        "   .   ": "切換任務隱私設定",
        "   -   ": "切換已完成任務的可見性",
        "  f(F) ": "修改（移除）任務期限",
        "   m   ": "移動任務",
        "   C   ": "從 Calcurse 匯入任務",
        }

MSG_NAME          = "CALCURE"
MSG_VIM           = "方向鍵和 Vim 快捷鍵（j、k、ZZ、ZQ）也可以使用！"
MSG_INFO          = "更多資訊，請造訪："
MSG_SITE          = "https://anufrievroman.gitbook.io/calcure"
MSG_KEYS_INFO     = "自訂快捷鍵，請參閱："
MSG_KEYS_SITE     = "https://anufrievroman.gitbook.io/calcure/keys"
MSG_EXIT          = "確定退出？(y/n)"

MSG_EVENT_HIGH    = "標記為高優先級行程編號："
MSG_EVENT_LOW     = "標記為低優先級行程編號："
MSG_EVENT_DONE    = "標記為已完成行程編號："
MSG_EVENT_RESET   = "重設行程狀態、編號："
MSG_EVENT_DEL     = "刪除行程編號："
MSG_EVENT_REN     = "重新命名行程編號："
MSG_NEW_TITLE     = "輸入新標題："
MSG_EVENT_MV      = "移動行程編號："
MSG_EVENT_MV_TO   = "移動行程至(YYYY/MM/DD)："
MSG_EVENT_MV_TO_D = "移動行程至："
MSG_EVENT_DATE    = "輸入日期："
MSG_EVENT_TITLE   = "輸入標題："
MSG_EVENT_REP     = "行程重複次數："
MSG_EVENT_FR      = "行程重複頻率：日(d)、週(w)、月(m)或年(y)？"
MSG_EVENT_IMP     = "從 Calcurse 匯入行程？(y/n)"
MSG_EVENT_PRIVACY = "切換行程編號隱私設定："
MSG_TM_ADD        = "為任務編號新增/暫停計時器："
MSG_TM_RESET      = "移除任務編號的計時器："
MSG_TS_HIGH       = "將任務編號標記為高優先級："
MSG_TS_LOW        = "將任務編號標記為低優先級："
MSG_TS_RES        = "重設任務編號狀態："
MSG_TS_DONE       = "將任務編號標記為已完成："
MSG_TS_DEL        = "刪除任務編號："
MSG_TS_DEL_ALL    = "確定刪除所有任務？(y/n)"
MSG_TS_EDT_ALL    = "確定執行此動作？(y/n)"
MSG_TS_MOVE       = "將任務從編號移動："
MSG_TS_MOVE_TO    = "移動任務至編號："
MSG_TS_EDIT       = "編輯任務編號："
MSG_TS_TOG        = "切換子任務編號："
MSG_TS_SUB        = "為任務編號新增子任務："
MSG_TS_TITLE      = "輸入子任務："
MSG_TS_IM         = "從 Calcurse 匯入任務？(y/n)"
MSG_TS_TW         = "從 Taskwarrior 匯入任務？(y/n)"
MSG_TS_NOTHING    = "沒有計劃..."
MSG_TS_DONE_COUNT = "已完成的任務"
MSG_TS_PRIVACY    = "切換任務編號隱私設定："
MSG_TS_DEAD_ADD   = "為任務編號新增截止日期："
MSG_TS_DEAD_DEL   = "移除任務編號的截止日期："
MSG_TS_DEAD_DATE  = "設定截止日期(YYYY/MM/DD)："
MSG_WEATHER       = "正在載入天氣..."
MSG_ERRORS        = "發生錯誤，詳情請查看配置資料夾中的 info.log。"
MSG_INPUT         = "輸入錯誤。"
MSG_GOTO          = "前往日期(YYYY/MM/DD)："
MSG_GOTO_D        = "前往日期："

CALENDAR_HINT     = "Space · 切換至日誌   a · 新增行程   n/p · 切換月份   ? · 所有快捷鍵"
CALENDAR_HINT_D   = "Space · 切換至日誌   a · 新增行程   n/p · 切換天數   ? · 所有快捷鍵"
CALENDAR_HINT_W   = "Space · 切換至日誌   a · 新增行程   n/p · 切換週數   ? · 所有快捷鍵"
JOURNAL_HINT      = "Space · 切換至日曆   a · 新增任務   d · 完成   i · 重要   ? · 所有快捷鍵"

DAYS = ["週一", "週二", "週三", "週四", "週五", "週六", "週日"]
DAYS_PERSIAN = ["SHANBEH", "YEKSHANBEH", "DOSHANBEH", "SESHANBEH", "CHAHARSHANBEH", "PANJSHANBEH", "JOMEH"]

MONTHS = ["一月", "二月", "三月", "四月", "五月", "六月", "七月", "八月", "九月", "十月", "十一月", "十二月"]
MONTHS_PERSIAN = ["FARVARDIN", "ORDIBEHESHT", "KHORDAD", "TIR", "MORDAD", "SHAHRIVAR", "MEHR", "ABAN", "AZAR", "DEY", "BAHMAN", "ESFAND"]



"""Slovene translations of the program interface"""

MSG_WELCOME_1 = "Dobrodošel/-a v Calcure"
MSG_WELCOME_2 = "Calcure je koledar in upravitelj opravil v terminalu!"
MSG_WELCOME_3 = "Datoteke s konfiguracijo in podatki so bile ustvarjene v:"
MSG_WELCOME_4 = "Za podporo, sodelovanje in dodatne informacije obišči:"
MSG_WELCOME_5 = "Pritisni ? za prikaz ukazov ali poljubno drugo tipko za nadaljevanje."

TITLE_KEYS_GENERAL = "SPLOŠNI UKAZI"
TITLE_KEYS_CALENDAR = "UPRAVLJANJE S KOLEDARJEM"
TITLE_KEYS_JOURNAL  = "UPRAVLJANJE Z DNEVNIKOM"

KEYS_GENERAL = {
        " Presl.": "Preklopi med koledarjem in dnevnikom",
        "   /   ": "Vklopi/izklopi razdeljeni zaslon",
        "   *   ": "Globalno vklopi/izklopi zasebnost",
        "   ?   ": "Prikaži/skrij pomoč",
        "   Q   ": "Osveži podatke",
        "   q   ": "Zapri program",
        }

KEYS_CALENDAR = {
        "  a(A) ": "Dodaj (ponavljajoč se) dogodek",
        "   n   ": "Naslednji mesec (dan)",
        "   p   ": "Prejšnji mesec (dan)",
        "   x   ": "Odstrani dogodek",
        "   r   ": "Preimenuj dogodek",
        "  m(M) ": "Premakni dogodek (v tem mescu)",
        "  g(G) ": "Pojdi na določen dan (v tem mescu)",
        "   v   ": "Preklopi na dnevni prikaz",
        "   w   ": "Preklopi na tedenski prikaz",
        "   W   ": "Prikaži/skrij številke tednov",
        "   h   ": "Označi/odznači dogodek kot visokoprioriteten",
        "   l   ": "Označi/odznači dogodek kot nizkoprioriteten",
        "   d   ": "Označi/odznači dogodek kot končan",
        "   .   ": "Vklopi/izklopi zasebnost za dogodek",
        "   C   ": "Uvozi dogodke iz programa Calcurse",
        "   R   ": "Vrni se na trenutni mesec (dan)",
        }

KEYS_TODO = {
        "  a(A) ": "Dodaj novo (pod)opravilo",
        "  h(H) ": "Označi/odznači eno (vsa) opravilo kot visokoprioritetno",
        "  l(L) ": "Označi/odznači eno (vsa) opravilo kot nizkoprioritetno",
        "  d(D) ": "Označi/odznači eno (vsa) opravilo kot opravljeno",
        "  u(U) ": "Odznači eno (vsa) opravilo",
        "  x(X) ": "Odstrani eno (vsa) opravilo (z vsemi podopravili)",
        "  t(T) ": "Poženi/zaustavi (odstrani) uro za opravilo",
        "   P   ": "Preklopi vse aktivne ure",
        "   r   ": "Preimenuj opravilo",
        "   s   ": "Spremeni opravilo v podopravilo ali obratno",
        "   .   ": "Vklopi/izklopi zasebnost za opravilo",
        "   -   ": "Vklopi/izklopi vidnost opravljenih opravil",
        "  f(F) ": "Spremeni (odstrani) rok za opravilo",
        "   m   ": "Premakni opravilo",
        "   C   ": "Uvozi opravila iz programa Calcurse",
        }

MSG_NAME          = "CALCURE"
MSG_VIM           = "Delujejo tudi puščice in Vimovi ukazi (j, k, ZZ, ZQ)!"
MSG_INFO          = "Več informacij najdeš na:"
MSG_SITE          = "https://anufrievroman.gitbook.io/calcure"
MSG_KEYS_INFO     = "Za prilagoditev tipk poglej:"
MSG_KEYS_SITE     = "https://anufrievroman.gitbook.io/calcure/keys"
MSG_EXIT          = "Res želiš zapreti program? (y/n) "

MSG_EVENT_HIGH    = "Označi/odznači kot visokoprioriteten dogodek s številko: "
MSG_EVENT_LOW     = "Označi/odznači kot nizkoprioriteten dogodek s številko: "
MSG_EVENT_DONE    = "Označi/odznači kot končan dogodek s številko: "
MSG_EVENT_RESET   = "Ponastavi stanje za dogodek s številko: "
MSG_EVENT_DEL     = "Odstrani dogodek s številko: "
MSG_EVENT_REN     = "Preimenuj dogodek s številko: "
MSG_NEW_TITLE     = "Vnesi novi naslov: "
MSG_EVENT_MV      = "Premakni dogodek s številko: "
MSG_EVENT_MV_TO   = "Premakni dogodek na (LLLL/MM/DD): "
MSG_EVENT_MV_TO_D = "Premakni dogodek na: "
MSG_EVENT_DATE    = "Vnesi datum: "
MSG_EVENT_TITLE   = "Vnesi naslov: "
MSG_EVENT_REP     = "Število ponovitev: "
MSG_EVENT_FR      = "Ponavljaj dogodek dnevno (d), tedensko (w), dvotedensko (b), mesečno (m) ali letno (y)? "
MSG_EVENT_IMP     = "Uvozi dogodke iz programa Calcurse? (y/n)"
MSG_EVENT_PRIVACY = "Vklopi/izklopi zasebnost za dogodek s številko: "
MSG_TM_ADD        = "Poženi/zaustavi uro za opravilo s številko: "
MSG_TM_RESET      = "Odstrani uro za opravilo s številko: "
MSG_TS_HIGH       = "Označi/odznači kot visokoprioritetno opravilo s številko: "
MSG_TS_LOW        = "Označi/odznači kot nizkoprioritetno opravilo s številko: "
MSG_TS_RES        = "Ponastavi stanje za opravilo s številko: "
MSG_TS_DONE       = "Označi/odznači kot opravljeno opravilo s številko: "
MSG_TS_DEL        = "Odstrani opravilo s številko: "
MSG_TS_DEL_ALL    = "Res želiš odstraniti vsa opravila? (y/n)"
MSG_TS_EDT_ALL    = "Potrjuješ to odločitev? (y/n)"
MSG_TS_MOVE       = "Premakni opravilo s številko: "
MSG_TS_MOVE_TO    = "Premakni opravilo na mesto s številko: "
MSG_TS_EDIT       = "Uredi opravilo s številko: "
MSG_TS_TOG        = "Spremeni opravilo/podopravilo s številko: "
MSG_TS_SUB        = "Dodaj podopravilo za opravilo s številko: "
MSG_TS_TITLE      = "Vnesi podopravilo: "
MSG_TS_IM         = "Uvozi opravila iz programa Calcurse? (y/n)"
MSG_TS_TW         = "Uvozi opravila iz programa Taskwarrior? (y/n)"
MSG_TS_NOTHING    = "Ni načrtov ..."
MSG_TS_DONE_COUNT = "Opravljena opravila"
MSG_TS_PRIVACY    = "Vklopi/izklopi zasebnost za opravilo s številko: "
MSG_TS_DEAD_ADD   = "Dodaj rok za opravilo s številko: "
MSG_TS_DEAD_DEL   = "Odstrani rok za opravilo s številko: "
MSG_TS_DEAD_DATE  = "Dodaj rok (LLLL/MM/DD): "
MSG_WEATHER       = "Vreme se nalaga ..."
MSG_ERRORS        = "Prišlo je do napak. Poglej info.log v svoji konfiguracijski mapi."
MSG_INPUT         = "Neveljaven vnos."
MSG_GOTO          = "Pojdi na datum (LLLL/MM/DD): "
MSG_GOTO_D        = "Pojdi na datum: "

CALENDAR_HINT     = "Presledek · Preklopi na dnevnik   a · Dodaj dogodek  n/p · Zamenjaj mesec   ? · Vsi ukazi"
CALENDAR_HINT_D   = "Presledek · Preklopi na dnevnik   a · Dodaj dogodek  n/p · Zamenjaj dan   ? · Vsi ukazi"
CALENDAR_HINT_W   = "Presledek · Preklopi na dnevnik   a · Dodaj dogodek  n/p · Zamenjaj teden   ? · Vsi ukazi"
JOURNAL_HINT      = "Presledek · Preklopi na koledar   a · Dodaj opravilo   d · Opravljeno   i · Pomembno   ? · Vsi ukazi"

DAYS = ["PONEDELJEK", "TOREK", "SREDA", "ČETRTEK", "PETEK", "SOBOTA", "NEDELJA"]
DAYS_PERSIAN = ["SHANBEH", "YEKSHANBEH", "DOSHANBEH", "SESHANBEH", "CHAHARSHANBEH", "PANJSHANBEH", "JOMEH"]

MONTHS = ["JANUAR", "FEBRUAR", "MAREC", "APRIL", "MAJ", "JUNIJ", "JULIJ", "AVGUST", "SEPTEMBER", "OKTOBER", "NOVEMBER", "DECEMBER"]
MONTHS_PERSIAN = ["FARVARDIN", "ORDIBEHESHT", "KHORDAD", "TIR", "MORDAD", "SHAHRIVAR", "MEHR", "ABAN", "AZAR", "DEY", "BAHMAN", "ESFAND"]



"""German translations of the program interface"""

MSG_WELCOME_1 = "Willkommen bei Calcure"
MSG_WELCOME_2 = "Ihr Terminal-Kalender und Aufgabenmanager!"
MSG_WELCOME_3 = "Konfigurations- und Datendateien wurden erstellt unter:"
MSG_WELCOME_4 = "Für Unterstützung, Beiträge und weitere Informationen besuchen Sie:"
MSG_WELCOME_5 = "Drücken Sie ? für alle Tastenkombinationen oder eine andere Taste zum Fortfahren."

TITLE_KEYS_GENERAL = "ALLGEMEINE TASTENKOMBINATIONEN"
TITLE_KEYS_CALENDAR = "KALENDER-TASTENKOMBINATIONEN"
TITLE_KEYS_JOURNAL  = "JOURNAL-TASTENKOMBINATIONEN"

KEYS_GENERAL = {
        " Space ": "Zwischen Kalender und Journal wechseln",
        "   /   ": "Geteilte Ansicht umschalten",
        "   *   ": "Globale Privatsphäre umschalten",
        "   ?   ": "Diese Hilfe ein-/ausblenden",
        "   Q   ": "Neu laden",
        "   q   ": "Beenden",
        }

KEYS_CALENDAR = {
        "  a(A) ": "Ereignis (wiederkehrend) hinzufügen",
        "   n   ": "Nächster Monat (Tag)",
        "   p   ": "Vorheriger Monat (Tag)",
        "   x   ": "Ereignis löschen",
        "   r   ": "Ereignis umbenennen",
        "  m(M) ": "Ereignis verschieben (in diesem Monat)",
        "  g(G) ": "Zu bestimmtem Tag gehen (in diesem Monat)",
        "   v   ": "Tagesübersicht umschalten",
        "   w   ": "Wochenübersicht umschalten",
        "   W   ": "Wochennummern ein-/ausblenden",
        "   h   ": "Ereignis als hohe Priorität markieren",
        "   l   ": "Ereignis als niedrige Priorität markieren",
        "   d   ": "Ereignis als erledigt markieren",
        "   .   ": "Privatsphäre des Ereignisses umschalten",
        "   C   ": "Ereignisse aus calcurse importieren",
        "   R   ": "Zum aktuellen Monat (Tag) zurückkehren",
        }

KEYS_TODO = {
        "  a(A) ": "Neue (Unter-)Aufgabe hinzufügen",
        "  h(H) ": "Eine (alle) Aufgaben als hohe Priorität markieren",
        "  l(L) ": "Eine (alle) Aufgaben als niedrige Priorität markieren",
        "  d(D) ": "Eine (alle) Aufgaben als erledigt markieren",
        "  u(U) ": "Markierung einer (aller) Aufgaben zurücksetzen",
        "  x(X) ": "Eine (alle) Aufgaben (inkl. Unteraufgaben) löschen",
        "  t(T) ": "Timer für Aufgabe starten/pausieren (entfernen)",
        "   P   ": "Alle aktiven Timer umschalten",
        "   r   ": "Aufgabe umbenennen",
        "   s   ": "Zwischen Aufgabe und Unteraufgabe wechseln",
        "   .   ": "Aufgaben-Privatsphäre umschalten",
        "   -   ": "Sichtbarkeit abgeschlossener Aufgaben umschalten",
        "  f(F) ": "Aufgaben-Deadline ändern (entfernen)",
        "   m   ": "Aufgabe verschieben",
        "   C   ": "Aufgaben aus calcurse importieren",
        }

MSG_NAME          = "CALCURE"
MSG_VIM           = "Pfeil- und Vim-Tasten (j, k, ZZ, ZQ) funktionieren ebenso!"
MSG_INFO          = "Für weitere Informationen besuchen Sie:"
MSG_SITE          = "https://anufrievroman.gitbook.io/calcure"
MSG_KEYS_INFO     = "Zur Anpassung der Tastenkürzel, siehe:"
MSG_KEYS_SITE     = "https://anufrievroman.gitbook.io/calcure/keys"
MSG_EXIT          = "Wirklich beenden? (j/n)"

MSG_EVENT_HIGH    = "Als hohe Priorität markieren, Ereignisnummer: "
MSG_EVENT_LOW     = "Als niedrige Priorität markieren, Ereignisnummer: "
MSG_EVENT_DONE    = "Als erledigt markieren, Ereignisnummer: "
MSG_EVENT_RESET   = "Status zurücksetzen für Ereignisnummer: "
MSG_EVENT_DEL     = "Ereignis löschen, Nummer: "
MSG_EVENT_REN     = "Ereignis umbenennen, Nummer: "
MSG_NEW_TITLE     = "Neuen Titel eingeben: "
MSG_EVENT_MV      = "Ereignis verschieben, Nummer: "
MSG_EVENT_MV_TO   = "Ereignis verschieben zu (JJJJ/MM/TT): "
MSG_EVENT_MV_TO_D = "Ereignis verschieben zu: "
MSG_EVENT_DATE    = "Datum eingeben: "
MSG_EVENT_TITLE   = "Titel eingeben: "
MSG_EVENT_REP     = "Wie oft soll das Ereignis wiederholt werden: "
MSG_EVENT_FR      = "Ereignis wiederholen jeden (d)Tag, (w)Woche, (b)zweiwöchentlich, (m)Monat oder (y)Jahr? "
MSG_EVENT_IMP     = "Ereignisse aus Calcurse importieren? (j/n)"
MSG_EVENT_PRIVACY = "Privatsphäre für Ereignisnummer umschalten: "
MSG_TM_ADD        = "Timer hinzufügen/pausieren für Aufgabennummer: "
MSG_TM_RESET      = "Timer für Aufgabennummer entfernen: "
MSG_TS_HIGH       = "Aufgabe als hohe Priorität markieren, Nummer: "
MSG_TS_LOW        = "Aufgabe als niedrige Priorität markieren, Nummer: "
MSG_TS_RES        = "Status für Aufgabennummer zurücksetzen: "
MSG_TS_DONE       = "Aufgabe als erledigt markieren, Nummer: "
MSG_TS_DEL        = "Aufgabe löschen, Nummer: "
MSG_TS_DEL_ALL    = "Wirklich alle Aufgaben löschen? (j/n)"
MSG_TS_EDT_ALL    = "Möchten Sie diese Aktion bestätigen? (j/n)"
MSG_TS_MOVE       = "Aufgabe verschieben von Nummer: "
MSG_TS_MOVE_TO    = "Aufgabe verschieben zu Nummer: "
MSG_TS_EDIT       = "Aufgabe bearbeiten, Nummer: "
MSG_TS_TOG        = "Unteraufgabe umschalten, Nummer: "
MSG_TS_SUB        = "Unteraufgabe für Aufgabennummer hinzufügen: "
MSG_TS_TITLE      = "Unteraufgabe eingeben: "
MSG_TS_IM         = "Aufgaben aus Calcurse importieren? (j/n)"
MSG_TS_TW         = "Aufgaben aus Taskwarrior importieren? (j/n)"
MSG_TS_NOTHING    = "Nichts geplant..."
MSG_TS_DONE_COUNT = "Aufgaben erledigt"
MSG_TS_PRIVACY    = "Privatsphäre für Aufgabennummer umschalten: "
MSG_TS_DEAD_ADD   = "Deadline für Aufgabennummer hinzufügen: "
MSG_TS_DEAD_DEL   = "Deadline für Aufgabennummer entfernen: "
MSG_TS_DEAD_DATE  = "Deadline hinzufügen am (JJJJ/MM/TT): "
MSG_WEATHER       = "Wetter wird geladen..."
MSG_ERRORS        = "Es sind Fehler aufgetreten. Siehe info.log im Konfigurationsordner."
MSG_INPUT         = "Ungültige Eingabe."
MSG_GOTO          = "Zum Datum gehen (JJJJ/MM/TT): "
MSG_GOTO_D        = "Zum Datum gehen: "

CALENDAR_HINT     = "Space · Zum Journal wechseln   a · Ereignis hinzufügen  n/p · Monat wechseln   ? · Alle Tastenkombinationen"
CALENDAR_HINT_D   = "Space · Zum Journal wechseln   a · Ereignis hinzufügen  n/p · Tag wechseln   ? · Alle Tastenkombinationen"
CALENDAR_HINT_W   = "Space · Zum Journal wechseln   a · Ereignis hinzufügen  n/p · Woche wechseln   ? · Alle Tastenkombinationen"
JOURNAL_HINT      = "Space · Zum Kalender wechseln   a · Aufgabe hinzufügen   d · Erledigt   i · Wichtig   ? · Alle Tastenkombinationen"

DAYS = ["MONTAG", "DIENSTAG", "MITTWOCH", "DONNERSTAG", "FREITAG", "SAMSTAG", "SONNTAG"]
DAYS_PERSIAN = ["SHANBEH", "YEKSHANBEH", "DOSHANBEH", "SESHANBEH", "CHAHARSHANBEH", "PANJSHANBEH", "JOMEH"]

MONTHS = ["JANUAR", "FEBRUAR", "MÄRZ", "APRIL", "MAI", "JUNI", "JULI", "AUGUST", "SEPTEMBER", "OKTOBER", "NOVEMBER", "DEZEMBER"]
MONTHS_PERSIAN = ["FARVARDIN", "ORDIBEHESHT", "KHORDAD", "TIR", "MORDAD", "SHAHRIVAR", "MEHR", "ABAN", "AZAR", "DEY", "BAHMAN", "ESFAND"]



"""Module that controls display of the moon phases"""

from math import cos, floor, fmod, modf, radians, sin


"""Most all equations are taken from the book Astronomical Algorithms,
   2nd Ed., by Jean Meeus, 1998.  """


def jd2ymd(jd: float) -> (int, int, float):
    """Converts the given Julian day number to the
    Gregorian year, month and day.

    The day is a float which represents the day and
    fraction of a day.
    For example, day = 1.5 would be 12 noon on the
    first day.

    Calculation follows pg. 63 of Astronomical Algorithms.
    No equation numbers given in text.  From Example 7.c:

    JD 2436116.31 => 1957 October 4.81

    """
    jd_plus = jd + 0.5
    F, Z = modf(jd_plus)
    if Z < 2299161:
        A = Z
    else:
        alpha = int((Z - 1867216.25) / 36524.25)
        A = Z + 1 + alpha - int(alpha / 4)
    B = A + 1524
    C = int((B - 122.1) / 365.25)
    D = int(365.25 * C)
    E = int((B - D) / 30.6001)
    day = B - D - int(30.6001 * E) + F
    if E < 14:
        month = E - 1
    else:
        month = E - 13
    if month > 2:
        year = C - 4716
    else:
        year = C - 4715

    return year, month, day


def delta_t(year: int) -> float:
    """Calculate the difference between Universal Time and
    Dynamic Time i.e., delta_t = Dynamic Time - Universal Time.
    The return value is in units of days.

    Using equations from NASA website:
    https://eclipse.gsfc.nasa.gov/SEhelp/deltatpoly2004.html
    Note this is a polynomial approximation.
    """
    if year > 2005 and year < 2050:
        t = year - 2000
        delta_t = 62.92 + 0.32217 * t + 0.005589 * t**2
    elif year >= 2050 and year < 2150:
        delta_t = -20 + 32 * ((year - 1820) / 100) ** 2 - 0.5628 * (2150 - year)
    else:
        delta_t = 0

    delta_t = delta_t / (60 * 60 * 24)  # Convert from seconds to days

    return delta_t


def is_divisible_by(dividend, divisor):
    """Determine if the given dividend is divisible (integer
    divivion) by the given divisor (without remainder).
    """
    return dividend % divisor == 0


def is_leap_year(year: int) -> bool:
    """Determine if the given year (Gregorian calendar)
    is a leap year or not.

    Calculation follows information given on pg. 62 of
    Astronomical Algorithms, in the box.  No equation
    numbers in text.
    """
    if is_divisible_by(year, 100):
        return is_divisible_by(year, 400)
    return is_divisible_by(year, 4)


def major_correction(k: float, T: float, phase: str) -> float:
    """Calculation of the large correction to the Julian Day
    obtained by Eq. 49.3.  The return value is in units of
    days.

    See pgs. 350 - 352 of Astronomical Algorithms.
    """
    E = 1.0 - 0.002516 * T - 0.000_007_4 * T**2  # Eq. 47.6
    M = (
        2.5534 + 29.105_356_7 * k - 0.000_001_4 * T**2 - 0.000_000_11 * T**3
    )  # Eq. 49.4, units: deg.
    M = fmod(M, 360)
    if M < 0:
        M += 360
    M = radians(M)
    M_prime = (
        201.5643
        + 385.816_935_28 * k
        + 0.010_758_2 * T**2
        + 0.000_012_38 * T**3
        - 0.000_000_058 * T**4
    )  # Eq. 49.5, units: deg.
    M_prime = fmod(M_prime, 360)
    if M_prime < 0:
        M_prime += 360
    M_prime = radians(M_prime)
    F = (
        160.7108
        + 390.670_502_84 * k
        - 0.001_611_8 * T**2
        - 0.000_002_27 * T**3
        + 0.000_000_011 * T**4
    )  # Eq. 49.6, units: deg
    F = fmod(F, 360)
    if F < 0:
        F += 360
    F = radians(F)
    Omega = (
        124.7746 - 1.563_755_88 * k + 0.002_067_2 * T**2 + 0.000_002_15 * T**3
    )  # Eq. 49.7, units: deg
    Omega = fmod(Omega, 360)
    if Omega < 0:
        Omega += 360
    Omega = radians(Omega)
    if phase == "New Moon":
        coeff = [
            -0.40720,
            0.17241,
            0.01608,
            0.01039,
            0.00739,
            -0.00514,
            0.00208,
            -0.00111,
            -0.00057,
            0.00056,
            -0.00042,
            0.00042,
            0.00038,
            -0.00024,
            -0.00017,
            -0.00007,
            0.00004,
            0.00004,
            0.00003,
            0.00003,
            -0.00003,
            0.00003,
            -0.00002,
            -0.00002,
            0.00002,
        ]
    if phase == "Full Moon":
        coeff = [
            -0.40614,
            0.17302,
            0.01614,
            0.01043,
            0.00734,
            -0.00515,
            0.00209,
            -0.00111,
            -0.00057,
            0.00056,
            -0.00042,
            0.00042,
            0.00038,
            -0.00024,
            -0.00017,
            -0.00007,
            0.00004,
            0.00004,
            0.00003,
            0.00003,
            -0.00003,
            0.00003,
            -0.00002,
            -0.00002,
            0.00002,
        ]
    if phase == "New Moon" or phase == "Full Moon":
        corr = (
            coeff[0] * sin(M_prime)
            + coeff[1] * E * sin(M)
            + coeff[2] * sin(2 * M_prime)
            + coeff[3] * sin(2 * F)
            + coeff[4] * E * sin(M_prime - M)
            + coeff[5] * E * sin(M_prime + M)
            + coeff[6] * E**2 * sin(2 * M)
            + coeff[7] * sin(M_prime - 2 * F)
            + coeff[8] * sin(M_prime + 2 * F)
            + coeff[9] * E * sin(2 * M_prime + M)
            + coeff[10] * sin(3 * M_prime)
            + coeff[11] * E * sin(M + 2 * F)
            + coeff[12] * E * sin(M - 2 * F)
            + coeff[13] * E * sin(2 * M_prime - M)
            + coeff[14] * sin(Omega)
            + coeff[15] * sin(M_prime + 2 * M)
            + coeff[16] * sin(2 * M_prime - 2 * F)
            + coeff[17] * sin(3 * M)
            + coeff[18] * sin(M_prime + M - 2 * F)
            + coeff[19] * sin(2 * M_prime + 2 * F)
            + coeff[20] * sin(M_prime + M + 2 * F)
            + coeff[21] * sin(M_prime - M + 2 * F)
            + coeff[22] * sin(M_prime - M - 2 * F)
            + coeff[23] * sin(3 * M_prime + M)
            + coeff[24] * sin(4 * M_prime)
        )
    if phase == "First Quarter" or phase == "Last Quarter":
        coeff = [
            -0.62801,
            0.17172,
            -0.01183,
            0.00862,
            0.00804,
            0.00454,
            0.00204,
            -0.00180,
            -0.00070,
            -0.00040,
            -0.00034,
            0.00032,
            0.00032,
            -0.00028,
            0.00027,
            -0.00017,
            -0.00005,
            0.00004,
            -0.00004,
            0.00004,
            0.00003,
            0.00003,
            0.00002,
            0.00002,
            -0.00002,
        ]
        corr = (
            coeff[0] * sin(M_prime)
            + coeff[1] * E * sin(M)
            + coeff[2] * E * sin(M_prime + M)
            + coeff[3] * sin(2 * M_prime)
            + coeff[4] * sin(2 * F)
            + coeff[5] * E * sin(M_prime - M)
            + coeff[6] * E**2 * sin(2 * M)
            + coeff[7] * sin(M_prime - 2 * F)
            + coeff[8] * sin(M_prime + 2 * F)
            + coeff[9] * sin(3 * M_prime)
            + coeff[10] * E * sin(2 * M_prime - M)
            + coeff[11] * E * sin(M + 2 * F)
            + coeff[12] * E * sin(M - 2 * F)
            + coeff[13] * E**2 * sin(M_prime + 2 * M)
            + coeff[14] * E * sin(2 * M_prime + M)
            + coeff[15] * sin(Omega)
            + coeff[16] * sin(M_prime - M - 2 * F)
            + coeff[17] * sin(2 * M_prime + 2 * F)
            + coeff[18] * sin(M_prime + M + 2 * F)
            + coeff[19] * sin(M_prime - 2 * M)
            + coeff[20] * sin(M_prime + M - 2 * F)
            + coeff[21] * sin(3 * M)
            + coeff[22] * sin(2 * M_prime - 2 * F)
            + coeff[23] * sin(M_prime - M + 2 * F)
            + coeff[24] * sin(3 * M_prime + M)
        )
        W = (
            0.00306
            - 0.00038 * E * cos(M)
            + 0.00026 * cos(M_prime)
            - 0.00002 * cos(M_prime - M)
            + 0.00002 * cos(M_prime + M)
            + 0.00002 * cos(2 * F)
        )
        if phase == "First Quarter":
            corr += W
        elif phase == "Last Quarter":
            corr -= W

    return corr


def minor_correction(k: float, T: float) -> float:
    """The small correction to the Julian Day calculated
    using Eq. 49.3.  The return value is in units of days.

    See pgs. 351 - 352 of Astronomical Algorithms.
    """
    a1 = 299.77 + 0.107408 * k - 0.009173 * T**2
    a2 = 251.88 + 0.016321 * k
    a3 = 251.83 + 26.651886 * k
    a4 = 349.42 + 36.412478 * k
    a5 = 84.66 + 18.206239 * k
    a6 = 141.74 + 53.303771 * k
    a7 = 207.14 + 2.453732 * k
    a8 = 154.84 + 7.306860 * k
    a9 = 34.52 + 27.261239 * k
    a10 = 207.19 + 0.121824 * k
    a11 = 291.34 + 1.844379 * k
    a12 = 161.72 + 24.198154 * k
    a13 = 239.56 + 25.513099 * k
    a14 = 331.55 + 3.592518 * k

    a1 = radians(fmod(a1, 360))
    a2 = radians(fmod(a2, 360))
    a3 = radians(fmod(a3, 360))
    a4 = radians(fmod(a4, 360))
    a5 = radians(fmod(a5, 360))
    a6 = radians(fmod(a6, 360))
    a7 = radians(fmod(a7, 360))
    a8 = radians(fmod(a8, 360))
    a9 = radians(fmod(a9, 360))
    a10 = radians(fmod(a10, 360))
    a11 = radians(fmod(a11, 360))
    a12 = radians(fmod(a12, 360))
    a13 = radians(fmod(a13, 360))
    a14 = radians(fmod(a14, 360))

    corr = (
        325 * sin(a1)
        + 165 * sin(a2)
        + 164 * sin(a3)
        + 126 * sin(a4)
        + 110 * sin(a5)
        + 62 * sin(a6)
        + 60 * sin(a7)
        + 56 * sin(a8)
        + 47 * sin(a9)
        + 42 * sin(a10)
        + 40 * sin(a11)
        + 37 * sin(a12)
        + 35 * sin(a13)
        + 23 * sin(a14)
    )
    corr = 1e-6 * corr

    return corr


def get_moon_phase(year: int, month: int, day: int) -> str:
    rval = ""
    if month in [1, 3, 5, 7, 8, 10, 12]:
        days_in_month = 31
    elif month in [4, 6, 9, 11]:
        days_in_month = 30
    else:
        if is_leap_year(year):
            days_in_month = 29
        else:
            days_in_month = 28
    year_dec = (
        year + (month - 1) / 12 + (day - 1) / (days_in_month * 12)
    )  # convert given year, month, and day to year with decimal
    k = (year_dec - 2000) * 12.3685  # Eq. 49.2
    k_new = floor(k)
    for i, k_val in enumerate([k_new, k_new + 0.25, k_new + 0.5, k_new + 0.75]):
        T = k_val / 1236.85  # Eq. 49.3
        jde_val = (
            2451550.09766
            + 29.530588861 * k_val
            + 0.00015437 * T**2
            - 0.00000015 * T**3
            + 0.000_000_000_73 * T**4
        )  # Eq. 49.1, units: days
        if i == 0:
            jde_val = (
                jde_val
                + major_correction(k_val, T, "New Moon")
                + minor_correction(k_val, T)
                - delta_t(year)
            )
        elif i == 1:
            jde_val = (
                jde_val
                + major_correction(k_val, T, "First Quarter")
                + minor_correction(k_val, T)
                - delta_t(year)
            )
        elif i == 2:
            jde_val = (
                jde_val
                + major_correction(k_val, T, "Full Moon")
                + minor_correction(k_val, T)
                - delta_t(year)
            )
        elif i == 3:
            jde_val = (
                jde_val
                + major_correction(k_val, T, "Last Quarter")
                + minor_correction(k_val, T)
                - delta_t(year)
            )
        calc_year, calc_month, calc_day = jd2ymd(jde_val)
        calc_day = int(calc_day)

        if (calc_year, calc_month, calc_day) == (year, month, day):
            if i == 0:
                rval = " 🌑"  # New Moon
            if i == 1:
                rval = " 🌓"  # First Quarter
            if i == 2:
                rval = " 🌕"  # Full Moon
            if i == 3:
                rval = " 🌗"  # Last Quarter
    return rval



"""Spanish translations of the program interface"""

MSG_WELCOME_1 = "Bienvenido a Calcure"
MSG_WELCOME_2 = "¡Tu calendario y administrador de tareas en la terminal!"
MSG_WELCOME_3 = "Se han creados archivos de configuración en:"
MSG_WELCOME_4 = "Para soporte, contribución e información adicional, visite:"
MSG_WELCOME_5 = "Presione ? para ver los atajos u otra tecla para continuar"

TITLE_KEYS_GENERAL = "ATAJOS GENERALES"
TITLE_KEYS_CALENDAR = "ATAJOS DEL CALENDARIO"
TITLE_KEYS_JOURNAL  = "ATAJOS DEL DIARIO"

KEYS_GENERAL = {
        " Espacio ": "Cambiar entre el calendario y el diario",
        "   /   ": "Alternar pantalla dividida",
        "   *   ": "Alternar privacidad global",
        "   ?   ": "Alternar para ayuda",
        "   Q   ": "Recargar",
        "   q   ": "Salir",
        }

KEYS_CALENDAR = {
        "  a(A) ": "Agregar un evento (recurrente)",
        "   n   ": "Próximo mes (día)",
        "   p   ": "Mes anterior (día)",
        "   x   ": "Eliminar un evento",
        "   r   ": "Renombrar un evento",
        "  m(M) ": "Mover evento (en este mes)",
        "  g(G) ": "Ir a un día específico (en este mes)",
        "   v   ": "Alternar vista diaria",
        "   w   ": "Alternar vista semanal",
        "   W   ": "Alternar números de semana",
        "   h   ": "Alternar evento como alta prioridad",
        "   l   ": "Alternar evento como baja prioridad",
        "   d   ": "Alternar evento como terminado",
        "   .   ": "Alternar la privacidad del evento",
        "   C   ": "Importar eventos desde calcurse",
        "   R   ": "Regresar al mes actual",
        }

KEYS_TODO = {
        "  a(A) ": "Agregar nueva (sub)tarea",
        "  h(H) ": "Alternar una (todas) de las tareas como alta prioridad",
        "  l(L) ": "Alternar una (todas) de las tareas como baja prioridad",
        "  d(D) ": "Alternar una (todas) de las tareas como terminadas",
        "  u(U) ": "Desmarcar una (todas) de las tareas",
        "  x(X) ": "Eliminar una (todas) de las tareas (con todas las subtareas)",
        "  t(T) ": "Iniciar/pausar (remover) el temporizador de una tarea",
        "   P   ": "Alternar todos los temporizadores activos",
        "   r   ": "Renombra una tarea",
        "   s   ": "Alternar entre tarea y subtarea",
        "   .   ": "Alternar la privacidad de la tarea",
        "   -   ": "Alternar visibilidad de tareas terminadas",
        "  f(F) ": "Cambiar (remover) la fecha límite de la tarea",
        "   m   ": "Mover una tarea",
        "   C   ": "Importar tareas desde calcurse",
        }

MSG_NAME          = "CALCURE"
MSG_VIM           = "¡Flechas y atajos de Vim (j, k, ZZ, ZQ) también funcionan!"
MSG_INFO          = "Para más información, visite:"
MSG_SITE          = "https://anufrievroman.gitbook.io/calcure"
MSG_KEYS_INFO     = "Para personalizar atajos, ver:"
MSG_KEYS_SITE     = "https://anufrievroman.gitbook.io/calcure/keys"
MSG_EXIT          = "¿Realmente quire salir? (y/n) "

MSG_EVENT_HIGH    = "Marcar como alta prioridad el evento número: ";
MSG_EVENT_LOW     = "Marcar como baja prioridad el evento número: "
MSG_EVENT_DONE    = "Marcar como terminado el evento número: "
MSG_EVENT_RESET   = "Resetear el estatus para el evento número: "
MSG_EVENT_DEL     = "Eliminar el evento número: "
MSG_EVENT_REN     = "Renombrar el evento número: "
MSG_NEW_TITLE     = "Ingresar un nuevo título: "
MSG_EVENT_MV      = "Mover evento número: "
MSG_EVENT_MV_TO   = "Mover evento a (YYYY/MM/DD): "
MSG_EVENT_MV_TO_D = "Mover evento a: "
MSG_EVENT_DATE    = "Ingresar la fecha: "
MSG_EVENT_TITLE   = "Ingresar un titulo: "
MSG_EVENT_REP     = "Cuantas veces quiere repetir el evento: "
MSG_EVENT_FR      = "¿Repetir el evento cada (d)ía, (w)semana, (b)quincenal, (m)mes o (y)año? "
MSG_EVENT_IMP     = "¿Importar eventos desde Calcurse? (y/n)"
MSG_EVENT_PRIVACY = "Alternar privacidad del evento número: "
MSG_TM_ADD        = "Agregar/pausar temporizador por número de tarea: "
MSG_TM_RESET      = "Remover temporizador para la tarea número: "
MSG_TS_HIGH       = "Marcar como alta prioridad la tarea número: "
MSG_TS_LOW        = "Marcar como baja prioridad la tarea número: "
MSG_TS_RES        = "Resetear el estatus para la tarea número: "
MSG_TS_DONE       = "Marcar como terminada la tarea número: "
MSG_TS_DEL        = "Eliminar la tarea número: "
MSG_TS_DEL_ALL    = "¿Realmente quiere eliminar todas las tareas? (y/n)"
MSG_TS_EDT_ALL    = "¿Confirmas esta acción? (y/n)"
MSG_TS_MOVE       = "Mover tarea desde el número: "
MSG_TS_MOVE_TO    = "Mover tarea al número: "
MSG_TS_EDIT       = "Editar tarea número: "
MSG_TS_TOG        = "Alternar subtarea número: "
MSG_TS_SUB        = "Agregar subtarea para la tarea número: "
MSG_TS_TITLE      = "Ingresar subtarea: "
MSG_TS_IM         = "¿Importar tareas desde Calcurse? (y/n)"
MSG_TS_TW         = "¿Importar tareas desde Taskwarrior? (y/n)"
MSG_TS_NOTHING    = "Nada planificado..."
MSG_TS_DONE_COUNT = "Tareas terminadas"
MSG_TS_PRIVACY    = "Alternar privacidad de la tarea número: "
MSG_TS_DEAD_ADD   = "Agregar fecha límite para la tarea número: "
MSG_TS_DEAD_DEL   = "Remover fecha límite para la tarea número: "
MSG_TS_DEAD_DATE  = "Agregar fecha límite en (YYY/MM/DD): "
MSG_WEATHER       = "Cargando el clima..."
MSG_ERRORS        = "Han ocurrido errores. Vea info.log en su carpeta config."
MSG_INPUT         = "Entrada incorrecta."
MSG_GOTO          = "Ir a la fecha (YYYY/MM/DD): "
MSG_GOTO_D        = "Ir a la fecha: "

CALENDAR_HINT     = "Espacio · Ir al diario   a · Agregar evento  n/p · Cambiar mes   ? · Mostrar atajos"
CALENDAR_HINT_D   = "Espacio · Ir al diario   a · Agregar evento  n/p · Cambiar día   ? · Mostrar atajos"
CALENDAR_HINT_W   = "Espacio · Ir al diario   a · Agregar evento  n/p · Cambiar semana   ? · Mostrar atajos"
JOURNAL_HINT      = "Espacio · Ir al calendario   a · Agregar tarea   d · Hecho   i · Importante   ? · Mostrar atajos"

DAYS = ["LUNES", "MARTES", "MIÉRCOLES", "JUEVES", "VIERNES", "SÁBADO", "DOMINGO"]
DAYS_PERSIAN = ["SHANBEH", "YEKSHANBEH", "DOSHANBEH", "SESHANBEH", "CHAHARSHANBEH", "PANJSHANBEH", "JOMEH"]

MONTHS = ["ENERO", "FEBRERO", "MARZO", "ABRIL", "MAYO", "JUNIO", "JULIO", "AGOSTO", "SEPTIEMBRE", "OCTUBRE", "NOVIEMBRE", "DICIEMBRE"]
MONTHS_PERSIAN = ["FARVARDIN", "ORDIBEHESHT", "KHORDAD", "TIR", "MORDAD", "SHAHRIVAR", "MEHR", "ABAN", "AZAR", "DEY", "BAHMAN", "ESFAND"]
